# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v19.resources.types import (
    asset_group_listing_group_filter as gagr_asset_group_listing_group_filter,
)
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "MutateAssetGroupListingGroupFiltersRequest",
        "AssetGroupListingGroupFilterOperation",
        "MutateAssetGroupListingGroupFiltersResponse",
        "MutateAssetGroupListingGroupFilterResult",
    },
)


class MutateAssetGroupListingGroupFiltersRequest(proto.Message):
    r"""Request message for
    [AssetGroupListingGroupFilterService.MutateAssetGroupListingGroupFilters][google.ads.googleads.v19.services.AssetGroupListingGroupFilterService.MutateAssetGroupListingGroupFilters].
    partial_failure is not supported because the tree needs to be
    validated together.

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose asset
            group listing group filters are being modified.
        operations (MutableSequence[google.ads.googleads.v19.services.types.AssetGroupListingGroupFilterOperation]):
            Required. The list of operations to perform
            on individual asset group listing group filters.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        response_content_type (google.ads.googleads.v19.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["AssetGroupListingGroupFilterOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="AssetGroupListingGroupFilterOperation",
        )
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class AssetGroupListingGroupFilterOperation(proto.Message):
    r"""A single operation (create, remove) on an asset group listing
    group filter.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v19.resources.types.AssetGroupListingGroupFilter):
            Create operation: No resource name is
            expected for the new asset group listing group
            filter.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v19.resources.types.AssetGroupListingGroupFilter):
            Update operation: The asset group listing
            group filter is expected to have a valid
            resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed asset
            group listing group filter is expected, in this format:
            ``customers/{customer_id}/assetGroupListingGroupFilters/{asset_group_id}~{listing_group_filter_id}``
            An entity can be removed only if it's not referenced by
            other parent_listing_group_id. If multiple entities are
            being deleted, the mutates must be in the correct order.

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    create: (
        gagr_asset_group_listing_group_filter.AssetGroupListingGroupFilter
    ) = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_asset_group_listing_group_filter.AssetGroupListingGroupFilter,
    )
    update: (
        gagr_asset_group_listing_group_filter.AssetGroupListingGroupFilter
    ) = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_asset_group_listing_group_filter.AssetGroupListingGroupFilter,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="operation",
    )


class MutateAssetGroupListingGroupFiltersResponse(proto.Message):
    r"""Response message for an asset group listing group filter
    mutate.

    Attributes:
        results (MutableSequence[google.ads.googleads.v19.services.types.MutateAssetGroupListingGroupFilterResult]):
            All results for the mutate.
    """

    results: MutableSequence["MutateAssetGroupListingGroupFilterResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="MutateAssetGroupListingGroupFilterResult",
        )
    )


class MutateAssetGroupListingGroupFilterResult(proto.Message):
    r"""The result for the asset group listing group filter mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
        asset_group_listing_group_filter (google.ads.googleads.v19.resources.types.AssetGroupListingGroupFilter):
            The mutated AssetGroupListingGroupFilter with only mutable
            fields after mutate. The field will only be returned when
            response_content_type is set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_group_listing_group_filter: (
        gagr_asset_group_listing_group_filter.AssetGroupListingGroupFilter
    ) = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_asset_group_listing_group_filter.AssetGroupListingGroupFilter,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .account_budget_proposal_service import (
    AccountBudgetProposalOperation,
    MutateAccountBudgetProposalRequest,
    MutateAccountBudgetProposalResponse,
    MutateAccountBudgetProposalResult,
)
from .account_link_service import (
    AccountLinkOperation,
    CreateAccountLinkRequest,
    CreateAccountLinkResponse,
    MutateAccountLinkRequest,
    MutateAccountLinkResponse,
    MutateAccountLinkResult,
)
from .ad_group_ad_label_service import (
    AdGroupAdLabelOperation,
    MutateAdGroupAdLabelResult,
    MutateAdGroupAdLabelsRequest,
    MutateAdGroupAdLabelsResponse,
)
from .ad_group_ad_service import (
    AdGroupAdOperation,
    AssetsWithFieldType,
    MutateAdGroupAdResult,
    MutateAdGroupAdsRequest,
    MutateAdGroupAdsResponse,
    RemoveAutomaticallyCreatedAssetsRequest,
)
from .ad_group_asset_service import (
    AdGroupAssetOperation,
    MutateAdGroupAssetResult,
    MutateAdGroupAssetsRequest,
    MutateAdGroupAssetsResponse,
)
from .ad_group_asset_set_service import (
    AdGroupAssetSetOperation,
    MutateAdGroupAssetSetResult,
    MutateAdGroupAssetSetsRequest,
    MutateAdGroupAssetSetsResponse,
)
from .ad_group_bid_modifier_service import (
    AdGroupBidModifierOperation,
    MutateAdGroupBidModifierResult,
    MutateAdGroupBidModifiersRequest,
    MutateAdGroupBidModifiersResponse,
)
from .ad_group_criterion_customizer_service import (
    AdGroupCriterionCustomizerOperation,
    MutateAdGroupCriterionCustomizerResult,
    MutateAdGroupCriterionCustomizersRequest,
    MutateAdGroupCriterionCustomizersResponse,
)
from .ad_group_criterion_label_service import (
    AdGroupCriterionLabelOperation,
    MutateAdGroupCriterionLabelResult,
    MutateAdGroupCriterionLabelsRequest,
    MutateAdGroupCriterionLabelsResponse,
)
from .ad_group_criterion_service import (
    AdGroupCriterionOperation,
    MutateAdGroupCriteriaRequest,
    MutateAdGroupCriteriaResponse,
    MutateAdGroupCriterionResult,
)
from .ad_group_customizer_service import (
    AdGroupCustomizerOperation,
    MutateAdGroupCustomizerResult,
    MutateAdGroupCustomizersRequest,
    MutateAdGroupCustomizersResponse,
)
from .ad_group_label_service import (
    AdGroupLabelOperation,
    MutateAdGroupLabelResult,
    MutateAdGroupLabelsRequest,
    MutateAdGroupLabelsResponse,
)
from .ad_group_service import (
    AdGroupOperation,
    MutateAdGroupResult,
    MutateAdGroupsRequest,
    MutateAdGroupsResponse,
)
from .ad_parameter_service import (
    AdParameterOperation,
    MutateAdParameterResult,
    MutateAdParametersRequest,
    MutateAdParametersResponse,
)
from .ad_service import (
    AdOperation,
    MutateAdResult,
    MutateAdsRequest,
    MutateAdsResponse,
)
from .asset_generation_service import (
    AssetGenerationExistingContext,
    FinalUrlImageGenerationInput,
    FreeformImageGenerationInput,
    GeneratedImage,
    GeneratedText,
    GenerateImagesRequest,
    GenerateImagesResponse,
    GenerateTextRequest,
    GenerateTextResponse,
    ProductRecontextGenerationImageInput,
    SourceImage,
)
from .asset_group_asset_service import (
    AssetGroupAssetOperation,
    MutateAssetGroupAssetResult,
    MutateAssetGroupAssetsRequest,
    MutateAssetGroupAssetsResponse,
)
from .asset_group_listing_group_filter_service import (
    AssetGroupListingGroupFilterOperation,
    MutateAssetGroupListingGroupFilterResult,
    MutateAssetGroupListingGroupFiltersRequest,
    MutateAssetGroupListingGroupFiltersResponse,
)
from .asset_group_service import (
    AssetGroupOperation,
    MutateAssetGroupResult,
    MutateAssetGroupsRequest,
    MutateAssetGroupsResponse,
)
from .asset_group_signal_service import (
    AssetGroupSignalOperation,
    MutateAssetGroupSignalResult,
    MutateAssetGroupSignalsRequest,
    MutateAssetGroupSignalsResponse,
)
from .asset_service import (
    AssetOperation,
    MutateAssetResult,
    MutateAssetsRequest,
    MutateAssetsResponse,
)
from .asset_set_asset_service import (
    AssetSetAssetOperation,
    MutateAssetSetAssetResult,
    MutateAssetSetAssetsRequest,
    MutateAssetSetAssetsResponse,
)
from .asset_set_service import (
    AssetSetOperation,
    MutateAssetSetResult,
    MutateAssetSetsRequest,
    MutateAssetSetsResponse,
)
from .audience_insights_service import (
    AudienceCompositionAttribute,
    AudienceCompositionAttributeCluster,
    AudienceCompositionMetrics,
    AudienceCompositionSection,
    AudienceOverlapItem,
    DimensionOverlapResult,
    GenerateAudienceCompositionInsightsRequest,
    GenerateAudienceCompositionInsightsResponse,
    GenerateAudienceOverlapInsightsRequest,
    GenerateAudienceOverlapInsightsResponse,
    GenerateInsightsFinderReportRequest,
    GenerateInsightsFinderReportResponse,
    GenerateSuggestedTargetingInsightsRequest,
    GenerateSuggestedTargetingInsightsResponse,
    GenerateTargetingSuggestionMetricsRequest,
    GenerateTargetingSuggestionMetricsResponse,
    InsightsAudience,
    InsightsAudienceAttributeGroup,
    InsightsAudienceDefinition,
    InsightsAudienceDescription,
    ListAudienceInsightsAttributesRequest,
    ListAudienceInsightsAttributesResponse,
    ListInsightsEligibleDatesRequest,
    ListInsightsEligibleDatesResponse,
    TargetingSuggestionMetrics,
)
from .audience_service import (
    AudienceOperation,
    MutateAudienceResult,
    MutateAudiencesRequest,
    MutateAudiencesResponse,
)
from .automatically_created_asset_removal_service import (
    RemoveCampaignAutomaticallyCreatedAssetOperation,
    RemoveCampaignAutomaticallyCreatedAssetRequest,
    RemoveCampaignAutomaticallyCreatedAssetResponse,
)
from .batch_job_service import (
    AddBatchJobOperationsRequest,
    AddBatchJobOperationsResponse,
    BatchJobOperation,
    BatchJobResult,
    ListBatchJobResultsRequest,
    ListBatchJobResultsResponse,
    MutateBatchJobRequest,
    MutateBatchJobResponse,
    MutateBatchJobResult,
    RunBatchJobRequest,
)
from .bidding_data_exclusion_service import (
    BiddingDataExclusionOperation,
    MutateBiddingDataExclusionsRequest,
    MutateBiddingDataExclusionsResponse,
    MutateBiddingDataExclusionsResult,
)
from .bidding_seasonality_adjustment_service import (
    BiddingSeasonalityAdjustmentOperation,
    MutateBiddingSeasonalityAdjustmentsRequest,
    MutateBiddingSeasonalityAdjustmentsResponse,
    MutateBiddingSeasonalityAdjustmentsResult,
)
from .bidding_strategy_service import (
    BiddingStrategyOperation,
    MutateBiddingStrategiesRequest,
    MutateBiddingStrategiesResponse,
    MutateBiddingStrategyResult,
)
from .billing_setup_service import (
    BillingSetupOperation,
    MutateBillingSetupRequest,
    MutateBillingSetupResponse,
    MutateBillingSetupResult,
)
from .brand_suggestion_service import (
    BrandSuggestion,
    SuggestBrandsRequest,
    SuggestBrandsResponse,
)
from .campaign_asset_service import (
    CampaignAssetOperation,
    MutateCampaignAssetResult,
    MutateCampaignAssetsRequest,
    MutateCampaignAssetsResponse,
)
from .campaign_asset_set_service import (
    CampaignAssetSetOperation,
    MutateCampaignAssetSetResult,
    MutateCampaignAssetSetsRequest,
    MutateCampaignAssetSetsResponse,
)
from .campaign_bid_modifier_service import (
    CampaignBidModifierOperation,
    MutateCampaignBidModifierResult,
    MutateCampaignBidModifiersRequest,
    MutateCampaignBidModifiersResponse,
)
from .campaign_budget_service import (
    CampaignBudgetOperation,
    MutateCampaignBudgetResult,
    MutateCampaignBudgetsRequest,
    MutateCampaignBudgetsResponse,
)
from .campaign_conversion_goal_service import (
    CampaignConversionGoalOperation,
    MutateCampaignConversionGoalResult,
    MutateCampaignConversionGoalsRequest,
    MutateCampaignConversionGoalsResponse,
)
from .campaign_criterion_service import (
    CampaignCriterionOperation,
    MutateCampaignCriteriaRequest,
    MutateCampaignCriteriaResponse,
    MutateCampaignCriterionResult,
)
from .campaign_customizer_service import (
    CampaignCustomizerOperation,
    MutateCampaignCustomizerResult,
    MutateCampaignCustomizersRequest,
    MutateCampaignCustomizersResponse,
)
from .campaign_draft_service import (
    CampaignDraftOperation,
    ListCampaignDraftAsyncErrorsRequest,
    ListCampaignDraftAsyncErrorsResponse,
    MutateCampaignDraftResult,
    MutateCampaignDraftsRequest,
    MutateCampaignDraftsResponse,
    PromoteCampaignDraftRequest,
)
from .campaign_goal_config_service import (
    CampaignGoalConfigOperation,
    MutateCampaignGoalConfigResult,
    MutateCampaignGoalConfigsRequest,
    MutateCampaignGoalConfigsResponse,
)
from .campaign_group_service import (
    CampaignGroupOperation,
    MutateCampaignGroupResult,
    MutateCampaignGroupsRequest,
    MutateCampaignGroupsResponse,
)
from .campaign_label_service import (
    CampaignLabelOperation,
    MutateCampaignLabelResult,
    MutateCampaignLabelsRequest,
    MutateCampaignLabelsResponse,
)
from .campaign_lifecycle_goal_service import (
    CampaignLifecycleGoalOperation,
    ConfigureCampaignLifecycleGoalsRequest,
    ConfigureCampaignLifecycleGoalsResponse,
    ConfigureCampaignLifecycleGoalsResult,
)
from .campaign_service import (
    BrandCampaignAssets,
    CampaignOperation,
    EnablementResult,
    EnableOperation,
    EnablePMaxBrandGuidelinesRequest,
    EnablePMaxBrandGuidelinesResponse,
    MutateCampaignResult,
    MutateCampaignsRequest,
    MutateCampaignsResponse,
)
from .campaign_shared_set_service import (
    CampaignSharedSetOperation,
    MutateCampaignSharedSetResult,
    MutateCampaignSharedSetsRequest,
    MutateCampaignSharedSetsResponse,
)
from .content_creator_insights_service import (
    GenerateCreatorInsightsRequest,
    GenerateCreatorInsightsResponse,
    GenerateTrendingInsightsRequest,
    GenerateTrendingInsightsResponse,
    SearchAudience,
    SearchTopics,
    TrendInsight,
    TrendInsightMetrics,
    YouTubeChannelInsights,
    YouTubeCreatorInsights,
    YouTubeMetrics,
)
from .conversion_action_service import (
    ConversionActionOperation,
    MutateConversionActionResult,
    MutateConversionActionsRequest,
    MutateConversionActionsResponse,
)
from .conversion_adjustment_upload_service import (
    ConversionAdjustment,
    ConversionAdjustmentResult,
    GclidDateTimePair,
    RestatementValue,
    UploadConversionAdjustmentsRequest,
    UploadConversionAdjustmentsResponse,
)
from .conversion_custom_variable_service import (
    ConversionCustomVariableOperation,
    MutateConversionCustomVariableResult,
    MutateConversionCustomVariablesRequest,
    MutateConversionCustomVariablesResponse,
)
from .conversion_goal_campaign_config_service import (
    ConversionGoalCampaignConfigOperation,
    MutateConversionGoalCampaignConfigResult,
    MutateConversionGoalCampaignConfigsRequest,
    MutateConversionGoalCampaignConfigsResponse,
)
from .conversion_upload_service import (
    CallConversion,
    CallConversionResult,
    CartData,
    ClickConversion,
    ClickConversionResult,
    CustomVariable,
    ExternalAttributionData,
    SessionAttributeKeyValuePair,
    SessionAttributesKeyValuePairs,
    UploadCallConversionsRequest,
    UploadCallConversionsResponse,
    UploadClickConversionsRequest,
    UploadClickConversionsResponse,
)
from .conversion_value_rule_service import (
    ConversionValueRuleOperation,
    MutateConversionValueRuleResult,
    MutateConversionValueRulesRequest,
    MutateConversionValueRulesResponse,
)
from .conversion_value_rule_set_service import (
    ConversionValueRuleSetOperation,
    MutateConversionValueRuleSetResult,
    MutateConversionValueRuleSetsRequest,
    MutateConversionValueRuleSetsResponse,
)
from .custom_audience_service import (
    CustomAudienceOperation,
    MutateCustomAudienceResult,
    MutateCustomAudiencesRequest,
    MutateCustomAudiencesResponse,
)
from .custom_conversion_goal_service import (
    CustomConversionGoalOperation,
    MutateCustomConversionGoalResult,
    MutateCustomConversionGoalsRequest,
    MutateCustomConversionGoalsResponse,
)
from .custom_interest_service import (
    CustomInterestOperation,
    MutateCustomInterestResult,
    MutateCustomInterestsRequest,
    MutateCustomInterestsResponse,
)
from .customer_asset_service import (
    CustomerAssetOperation,
    MutateCustomerAssetResult,
    MutateCustomerAssetsRequest,
    MutateCustomerAssetsResponse,
)
from .customer_asset_set_service import (
    CustomerAssetSetOperation,
    MutateCustomerAssetSetResult,
    MutateCustomerAssetSetsRequest,
    MutateCustomerAssetSetsResponse,
)
from .customer_client_link_service import (
    CustomerClientLinkOperation,
    MutateCustomerClientLinkRequest,
    MutateCustomerClientLinkResponse,
    MutateCustomerClientLinkResult,
)
from .customer_conversion_goal_service import (
    CustomerConversionGoalOperation,
    MutateCustomerConversionGoalResult,
    MutateCustomerConversionGoalsRequest,
    MutateCustomerConversionGoalsResponse,
)
from .customer_customizer_service import (
    CustomerCustomizerOperation,
    MutateCustomerCustomizerResult,
    MutateCustomerCustomizersRequest,
    MutateCustomerCustomizersResponse,
)
from .customer_label_service import (
    CustomerLabelOperation,
    MutateCustomerLabelResult,
    MutateCustomerLabelsRequest,
    MutateCustomerLabelsResponse,
)
from .customer_lifecycle_goal_service import (
    ConfigureCustomerLifecycleGoalsRequest,
    ConfigureCustomerLifecycleGoalsResponse,
    ConfigureCustomerLifecycleGoalsResult,
    CustomerLifecycleGoalOperation,
)
from .customer_manager_link_service import (
    CustomerManagerLinkOperation,
    MoveManagerLinkRequest,
    MoveManagerLinkResponse,
    MutateCustomerManagerLinkRequest,
    MutateCustomerManagerLinkResponse,
    MutateCustomerManagerLinkResult,
)
from .customer_negative_criterion_service import (
    CustomerNegativeCriterionOperation,
    MutateCustomerNegativeCriteriaRequest,
    MutateCustomerNegativeCriteriaResponse,
    MutateCustomerNegativeCriteriaResult,
)
from .customer_service import (
    CreateCustomerClientRequest,
    CreateCustomerClientResponse,
    CustomerOperation,
    ListAccessibleCustomersRequest,
    ListAccessibleCustomersResponse,
    MutateCustomerRequest,
    MutateCustomerResponse,
    MutateCustomerResult,
)
from .customer_sk_ad_network_conversion_value_schema_service import (
    CustomerSkAdNetworkConversionValueSchemaOperation,
    MutateCustomerSkAdNetworkConversionValueSchemaRequest,
    MutateCustomerSkAdNetworkConversionValueSchemaResponse,
    MutateCustomerSkAdNetworkConversionValueSchemaResult,
)
from .customer_user_access_invitation_service import (
    CustomerUserAccessInvitationOperation,
    MutateCustomerUserAccessInvitationRequest,
    MutateCustomerUserAccessInvitationResponse,
    MutateCustomerUserAccessInvitationResult,
)
from .customer_user_access_service import (
    CustomerUserAccessOperation,
    MutateCustomerUserAccessRequest,
    MutateCustomerUserAccessResponse,
    MutateCustomerUserAccessResult,
)
from .customizer_attribute_service import (
    CustomizerAttributeOperation,
    MutateCustomizerAttributeResult,
    MutateCustomizerAttributesRequest,
    MutateCustomizerAttributesResponse,
)
from .data_link_service import (
    CreateDataLinkRequest,
    CreateDataLinkResponse,
    RemoveDataLinkRequest,
    RemoveDataLinkResponse,
    UpdateDataLinkRequest,
    UpdateDataLinkResponse,
)
from .experiment_arm_service import (
    ExperimentArmOperation,
    MutateExperimentArmResult,
    MutateExperimentArmsRequest,
    MutateExperimentArmsResponse,
)
from .experiment_service import (
    CampaignBudgetMapping,
    EndExperimentRequest,
    ExperimentOperation,
    GraduateExperimentRequest,
    ListExperimentAsyncErrorsRequest,
    ListExperimentAsyncErrorsResponse,
    MutateExperimentResult,
    MutateExperimentsRequest,
    MutateExperimentsResponse,
    PromoteExperimentMetadata,
    PromoteExperimentRequest,
    ScheduleExperimentMetadata,
    ScheduleExperimentRequest,
)
from .geo_target_constant_service import (
    GeoTargetConstantSuggestion,
    SuggestGeoTargetConstantsRequest,
    SuggestGeoTargetConstantsResponse,
)
from .goal_service import (
    GoalOperation,
    MutateGoalResult,
    MutateGoalsRequest,
    MutateGoalsResponse,
)
from .google_ads_field_service import (
    GetGoogleAdsFieldRequest,
    SearchGoogleAdsFieldsRequest,
    SearchGoogleAdsFieldsResponse,
)
from .google_ads_service import (
    GoogleAdsRow,
    MutateGoogleAdsRequest,
    MutateGoogleAdsResponse,
    MutateOperation,
    MutateOperationResponse,
    SearchGoogleAdsRequest,
    SearchGoogleAdsResponse,
    SearchGoogleAdsStreamRequest,
    SearchGoogleAdsStreamResponse,
    SearchSettings,
)
from .identity_verification_service import (
    GetIdentityVerificationRequest,
    GetIdentityVerificationResponse,
    IdentityVerification,
    IdentityVerificationProgress,
    IdentityVerificationRequirement,
    StartIdentityVerificationRequest,
)
from .invoice_service import (
    ListInvoicesRequest,
    ListInvoicesResponse,
)
from .keyword_plan_ad_group_keyword_service import (
    KeywordPlanAdGroupKeywordOperation,
    MutateKeywordPlanAdGroupKeywordResult,
    MutateKeywordPlanAdGroupKeywordsRequest,
    MutateKeywordPlanAdGroupKeywordsResponse,
)
from .keyword_plan_ad_group_service import (
    KeywordPlanAdGroupOperation,
    MutateKeywordPlanAdGroupResult,
    MutateKeywordPlanAdGroupsRequest,
    MutateKeywordPlanAdGroupsResponse,
)
from .keyword_plan_campaign_keyword_service import (
    KeywordPlanCampaignKeywordOperation,
    MutateKeywordPlanCampaignKeywordResult,
    MutateKeywordPlanCampaignKeywordsRequest,
    MutateKeywordPlanCampaignKeywordsResponse,
)
from .keyword_plan_campaign_service import (
    KeywordPlanCampaignOperation,
    MutateKeywordPlanCampaignResult,
    MutateKeywordPlanCampaignsRequest,
    MutateKeywordPlanCampaignsResponse,
)
from .keyword_plan_idea_service import (
    AdGroupKeywordSuggestion,
    BiddableKeyword,
    CampaignToForecast,
    CriterionBidModifier,
    ForecastAdGroup,
    GenerateAdGroupThemesRequest,
    GenerateAdGroupThemesResponse,
    GenerateKeywordForecastMetricsRequest,
    GenerateKeywordForecastMetricsResponse,
    GenerateKeywordHistoricalMetricsRequest,
    GenerateKeywordHistoricalMetricsResponse,
    GenerateKeywordHistoricalMetricsResult,
    GenerateKeywordIdeaResponse,
    GenerateKeywordIdeaResult,
    GenerateKeywordIdeasRequest,
    KeywordAndUrlSeed,
    KeywordForecastMetrics,
    KeywordSeed,
    ManualCpcBiddingStrategy,
    MaximizeClicksBiddingStrategy,
    MaximizeConversionsBiddingStrategy,
    SiteSeed,
    UnusableAdGroup,
    UrlSeed,
)
from .keyword_plan_service import (
    KeywordPlanOperation,
    MutateKeywordPlansRequest,
    MutateKeywordPlansResponse,
    MutateKeywordPlansResult,
)
from .keyword_theme_constant_service import (
    SuggestKeywordThemeConstantsRequest,
    SuggestKeywordThemeConstantsResponse,
)
from .label_service import (
    LabelOperation,
    MutateLabelResult,
    MutateLabelsRequest,
    MutateLabelsResponse,
)
from .local_services_lead_service import (
    AppendLeadConversationRequest,
    AppendLeadConversationResponse,
    Conversation,
    ConversationOrError,
    ProvideLeadFeedbackRequest,
    ProvideLeadFeedbackResponse,
    SurveyDissatisfied,
    SurveySatisfied,
)
from .offline_user_data_job_service import (
    AddOfflineUserDataJobOperationsRequest,
    AddOfflineUserDataJobOperationsResponse,
    CreateOfflineUserDataJobRequest,
    CreateOfflineUserDataJobResponse,
    OfflineUserDataJobOperation,
    RunOfflineUserDataJobRequest,
)
from .payments_account_service import (
    ListPaymentsAccountsRequest,
    ListPaymentsAccountsResponse,
)
from .product_link_invitation_service import (
    CreateProductLinkInvitationRequest,
    CreateProductLinkInvitationResponse,
    RemoveProductLinkInvitationRequest,
    RemoveProductLinkInvitationResponse,
    UpdateProductLinkInvitationRequest,
    UpdateProductLinkInvitationResponse,
)
from .product_link_service import (
    CreateProductLinkRequest,
    CreateProductLinkResponse,
    RemoveProductLinkRequest,
    RemoveProductLinkResponse,
)
from .reach_plan_service import (
    AdvancedProductTargeting,
    AudienceTargeting,
    CampaignDuration,
    ConversionRateSuggestion,
    EffectiveFrequencyBreakdown,
    EffectiveFrequencyLimit,
    Forecast,
    ForecastMetricOptions,
    FrequencyCap,
    GenerateConversionRatesRequest,
    GenerateConversionRatesResponse,
    GenerateReachForecastRequest,
    GenerateReachForecastResponse,
    ListPlannableLocationsRequest,
    ListPlannableLocationsResponse,
    ListPlannableProductsRequest,
    ListPlannableProductsResponse,
    ListPlannableUserInterestsRequest,
    ListPlannableUserInterestsResponse,
    ListPlannableUserListsRequest,
    ListPlannableUserListsResponse,
    OnTargetAudienceMetrics,
    PlannableLocation,
    PlannableTargeting,
    PlannableUserInterest,
    PlannableUserList,
    PlannableUserListMetadata,
    PlannedProduct,
    PlannedProductForecast,
    PlannedProductReachForecast,
    ProductMetadata,
    ReachCurve,
    ReachForecast,
    SurfaceTargeting,
    SurfaceTargetingCombinations,
    TargetFrequencySettings,
    Targeting,
    YouTubeSelectLineUp,
    YouTubeSelectSettings,
)
from .recommendation_service import (
    ApplyRecommendationOperation,
    ApplyRecommendationRequest,
    ApplyRecommendationResponse,
    ApplyRecommendationResult,
    DismissRecommendationRequest,
    DismissRecommendationResponse,
    GenerateRecommendationsRequest,
    GenerateRecommendationsResponse,
)
from .recommendation_subscription_service import (
    MutateRecommendationSubscriptionRequest,
    MutateRecommendationSubscriptionResponse,
    MutateRecommendationSubscriptionResult,
    RecommendationSubscriptionOperation,
)
from .remarketing_action_service import (
    MutateRemarketingActionResult,
    MutateRemarketingActionsRequest,
    MutateRemarketingActionsResponse,
    RemarketingActionOperation,
)
from .shareable_preview_service import (
    AssetGroupIdentifier,
    GenerateShareablePreviewsRequest,
    GenerateShareablePreviewsResponse,
    ShareablePreview,
    ShareablePreviewOrError,
    ShareablePreviewResult,
)
from .shared_criterion_service import (
    MutateSharedCriteriaRequest,
    MutateSharedCriteriaResponse,
    MutateSharedCriterionResult,
    SharedCriterionOperation,
)
from .shared_set_service import (
    MutateSharedSetResult,
    MutateSharedSetsRequest,
    MutateSharedSetsResponse,
    SharedSetOperation,
)
from .smart_campaign_setting_service import (
    GetSmartCampaignStatusRequest,
    GetSmartCampaignStatusResponse,
    MutateSmartCampaignSettingResult,
    MutateSmartCampaignSettingsRequest,
    MutateSmartCampaignSettingsResponse,
    SmartCampaignEligibleDetails,
    SmartCampaignEndedDetails,
    SmartCampaignNotEligibleDetails,
    SmartCampaignPausedDetails,
    SmartCampaignRemovedDetails,
    SmartCampaignSettingOperation,
)
from .smart_campaign_suggest_service import (
    SmartCampaignSuggestionInfo,
    SuggestKeywordThemesRequest,
    SuggestKeywordThemesResponse,
    SuggestSmartCampaignAdRequest,
    SuggestSmartCampaignAdResponse,
    SuggestSmartCampaignBudgetOptionsRequest,
    SuggestSmartCampaignBudgetOptionsResponse,
)
from .third_party_app_analytics_link_service import (
    RegenerateShareableLinkIdRequest,
    RegenerateShareableLinkIdResponse,
)
from .travel_asset_suggestion_service import (
    HotelAssetSuggestion,
    HotelImageAsset,
    HotelTextAsset,
    SuggestTravelAssetsRequest,
    SuggestTravelAssetsResponse,
)
from .user_data_service import (
    UploadUserDataRequest,
    UploadUserDataResponse,
    UserDataOperation,
)
from .user_list_customer_type_service import (
    MutateUserListCustomerTypeResult,
    MutateUserListCustomerTypesRequest,
    MutateUserListCustomerTypesResponse,
    UserListCustomerTypeOperation,
)
from .user_list_service import (
    MutateUserListResult,
    MutateUserListsRequest,
    MutateUserListsResponse,
    UserListOperation,
)

__all__ = (
    "AccountBudgetProposalOperation",
    "MutateAccountBudgetProposalRequest",
    "MutateAccountBudgetProposalResponse",
    "MutateAccountBudgetProposalResult",
    "AccountLinkOperation",
    "CreateAccountLinkRequest",
    "CreateAccountLinkResponse",
    "MutateAccountLinkRequest",
    "MutateAccountLinkResponse",
    "MutateAccountLinkResult",
    "AdGroupAdLabelOperation",
    "MutateAdGroupAdLabelResult",
    "MutateAdGroupAdLabelsRequest",
    "MutateAdGroupAdLabelsResponse",
    "AdGroupAdOperation",
    "AssetsWithFieldType",
    "MutateAdGroupAdResult",
    "MutateAdGroupAdsRequest",
    "MutateAdGroupAdsResponse",
    "RemoveAutomaticallyCreatedAssetsRequest",
    "AdGroupAssetOperation",
    "MutateAdGroupAssetResult",
    "MutateAdGroupAssetsRequest",
    "MutateAdGroupAssetsResponse",
    "AdGroupAssetSetOperation",
    "MutateAdGroupAssetSetResult",
    "MutateAdGroupAssetSetsRequest",
    "MutateAdGroupAssetSetsResponse",
    "AdGroupBidModifierOperation",
    "MutateAdGroupBidModifierResult",
    "MutateAdGroupBidModifiersRequest",
    "MutateAdGroupBidModifiersResponse",
    "AdGroupCriterionCustomizerOperation",
    "MutateAdGroupCriterionCustomizerResult",
    "MutateAdGroupCriterionCustomizersRequest",
    "MutateAdGroupCriterionCustomizersResponse",
    "AdGroupCriterionLabelOperation",
    "MutateAdGroupCriterionLabelResult",
    "MutateAdGroupCriterionLabelsRequest",
    "MutateAdGroupCriterionLabelsResponse",
    "AdGroupCriterionOperation",
    "MutateAdGroupCriteriaRequest",
    "MutateAdGroupCriteriaResponse",
    "MutateAdGroupCriterionResult",
    "AdGroupCustomizerOperation",
    "MutateAdGroupCustomizerResult",
    "MutateAdGroupCustomizersRequest",
    "MutateAdGroupCustomizersResponse",
    "AdGroupLabelOperation",
    "MutateAdGroupLabelResult",
    "MutateAdGroupLabelsRequest",
    "MutateAdGroupLabelsResponse",
    "AdGroupOperation",
    "MutateAdGroupResult",
    "MutateAdGroupsRequest",
    "MutateAdGroupsResponse",
    "AdParameterOperation",
    "MutateAdParameterResult",
    "MutateAdParametersRequest",
    "MutateAdParametersResponse",
    "AdOperation",
    "MutateAdResult",
    "MutateAdsRequest",
    "MutateAdsResponse",
    "AssetGenerationExistingContext",
    "FinalUrlImageGenerationInput",
    "FreeformImageGenerationInput",
    "GeneratedImage",
    "GeneratedText",
    "GenerateImagesRequest",
    "GenerateImagesResponse",
    "GenerateTextRequest",
    "GenerateTextResponse",
    "ProductRecontextGenerationImageInput",
    "SourceImage",
    "AssetGroupAssetOperation",
    "MutateAssetGroupAssetResult",
    "MutateAssetGroupAssetsRequest",
    "MutateAssetGroupAssetsResponse",
    "AssetGroupListingGroupFilterOperation",
    "MutateAssetGroupListingGroupFilterResult",
    "MutateAssetGroupListingGroupFiltersRequest",
    "MutateAssetGroupListingGroupFiltersResponse",
    "AssetGroupOperation",
    "MutateAssetGroupResult",
    "MutateAssetGroupsRequest",
    "MutateAssetGroupsResponse",
    "AssetGroupSignalOperation",
    "MutateAssetGroupSignalResult",
    "MutateAssetGroupSignalsRequest",
    "MutateAssetGroupSignalsResponse",
    "AssetOperation",
    "MutateAssetResult",
    "MutateAssetsRequest",
    "MutateAssetsResponse",
    "AssetSetAssetOperation",
    "MutateAssetSetAssetResult",
    "MutateAssetSetAssetsRequest",
    "MutateAssetSetAssetsResponse",
    "AssetSetOperation",
    "MutateAssetSetResult",
    "MutateAssetSetsRequest",
    "MutateAssetSetsResponse",
    "AudienceCompositionAttribute",
    "AudienceCompositionAttributeCluster",
    "AudienceCompositionMetrics",
    "AudienceCompositionSection",
    "AudienceOverlapItem",
    "DimensionOverlapResult",
    "GenerateAudienceCompositionInsightsRequest",
    "GenerateAudienceCompositionInsightsResponse",
    "GenerateAudienceOverlapInsightsRequest",
    "GenerateAudienceOverlapInsightsResponse",
    "GenerateInsightsFinderReportRequest",
    "GenerateInsightsFinderReportResponse",
    "GenerateSuggestedTargetingInsightsRequest",
    "GenerateSuggestedTargetingInsightsResponse",
    "GenerateTargetingSuggestionMetricsRequest",
    "GenerateTargetingSuggestionMetricsResponse",
    "InsightsAudience",
    "InsightsAudienceAttributeGroup",
    "InsightsAudienceDefinition",
    "InsightsAudienceDescription",
    "ListAudienceInsightsAttributesRequest",
    "ListAudienceInsightsAttributesResponse",
    "ListInsightsEligibleDatesRequest",
    "ListInsightsEligibleDatesResponse",
    "TargetingSuggestionMetrics",
    "AudienceOperation",
    "MutateAudienceResult",
    "MutateAudiencesRequest",
    "MutateAudiencesResponse",
    "RemoveCampaignAutomaticallyCreatedAssetOperation",
    "RemoveCampaignAutomaticallyCreatedAssetRequest",
    "RemoveCampaignAutomaticallyCreatedAssetResponse",
    "AddBatchJobOperationsRequest",
    "AddBatchJobOperationsResponse",
    "BatchJobOperation",
    "BatchJobResult",
    "ListBatchJobResultsRequest",
    "ListBatchJobResultsResponse",
    "MutateBatchJobRequest",
    "MutateBatchJobResponse",
    "MutateBatchJobResult",
    "RunBatchJobRequest",
    "BiddingDataExclusionOperation",
    "MutateBiddingDataExclusionsRequest",
    "MutateBiddingDataExclusionsResponse",
    "MutateBiddingDataExclusionsResult",
    "BiddingSeasonalityAdjustmentOperation",
    "MutateBiddingSeasonalityAdjustmentsRequest",
    "MutateBiddingSeasonalityAdjustmentsResponse",
    "MutateBiddingSeasonalityAdjustmentsResult",
    "BiddingStrategyOperation",
    "MutateBiddingStrategiesRequest",
    "MutateBiddingStrategiesResponse",
    "MutateBiddingStrategyResult",
    "BillingSetupOperation",
    "MutateBillingSetupRequest",
    "MutateBillingSetupResponse",
    "MutateBillingSetupResult",
    "BrandSuggestion",
    "SuggestBrandsRequest",
    "SuggestBrandsResponse",
    "CampaignAssetOperation",
    "MutateCampaignAssetResult",
    "MutateCampaignAssetsRequest",
    "MutateCampaignAssetsResponse",
    "CampaignAssetSetOperation",
    "MutateCampaignAssetSetResult",
    "MutateCampaignAssetSetsRequest",
    "MutateCampaignAssetSetsResponse",
    "CampaignBidModifierOperation",
    "MutateCampaignBidModifierResult",
    "MutateCampaignBidModifiersRequest",
    "MutateCampaignBidModifiersResponse",
    "CampaignBudgetOperation",
    "MutateCampaignBudgetResult",
    "MutateCampaignBudgetsRequest",
    "MutateCampaignBudgetsResponse",
    "CampaignConversionGoalOperation",
    "MutateCampaignConversionGoalResult",
    "MutateCampaignConversionGoalsRequest",
    "MutateCampaignConversionGoalsResponse",
    "CampaignCriterionOperation",
    "MutateCampaignCriteriaRequest",
    "MutateCampaignCriteriaResponse",
    "MutateCampaignCriterionResult",
    "CampaignCustomizerOperation",
    "MutateCampaignCustomizerResult",
    "MutateCampaignCustomizersRequest",
    "MutateCampaignCustomizersResponse",
    "CampaignDraftOperation",
    "ListCampaignDraftAsyncErrorsRequest",
    "ListCampaignDraftAsyncErrorsResponse",
    "MutateCampaignDraftResult",
    "MutateCampaignDraftsRequest",
    "MutateCampaignDraftsResponse",
    "PromoteCampaignDraftRequest",
    "CampaignGoalConfigOperation",
    "MutateCampaignGoalConfigResult",
    "MutateCampaignGoalConfigsRequest",
    "MutateCampaignGoalConfigsResponse",
    "CampaignGroupOperation",
    "MutateCampaignGroupResult",
    "MutateCampaignGroupsRequest",
    "MutateCampaignGroupsResponse",
    "CampaignLabelOperation",
    "MutateCampaignLabelResult",
    "MutateCampaignLabelsRequest",
    "MutateCampaignLabelsResponse",
    "CampaignLifecycleGoalOperation",
    "ConfigureCampaignLifecycleGoalsRequest",
    "ConfigureCampaignLifecycleGoalsResponse",
    "ConfigureCampaignLifecycleGoalsResult",
    "BrandCampaignAssets",
    "CampaignOperation",
    "EnablementResult",
    "EnableOperation",
    "EnablePMaxBrandGuidelinesRequest",
    "EnablePMaxBrandGuidelinesResponse",
    "MutateCampaignResult",
    "MutateCampaignsRequest",
    "MutateCampaignsResponse",
    "CampaignSharedSetOperation",
    "MutateCampaignSharedSetResult",
    "MutateCampaignSharedSetsRequest",
    "MutateCampaignSharedSetsResponse",
    "GenerateCreatorInsightsRequest",
    "GenerateCreatorInsightsResponse",
    "GenerateTrendingInsightsRequest",
    "GenerateTrendingInsightsResponse",
    "SearchAudience",
    "SearchTopics",
    "TrendInsight",
    "TrendInsightMetrics",
    "YouTubeChannelInsights",
    "YouTubeCreatorInsights",
    "YouTubeMetrics",
    "ConversionActionOperation",
    "MutateConversionActionResult",
    "MutateConversionActionsRequest",
    "MutateConversionActionsResponse",
    "ConversionAdjustment",
    "ConversionAdjustmentResult",
    "GclidDateTimePair",
    "RestatementValue",
    "UploadConversionAdjustmentsRequest",
    "UploadConversionAdjustmentsResponse",
    "ConversionCustomVariableOperation",
    "MutateConversionCustomVariableResult",
    "MutateConversionCustomVariablesRequest",
    "MutateConversionCustomVariablesResponse",
    "ConversionGoalCampaignConfigOperation",
    "MutateConversionGoalCampaignConfigResult",
    "MutateConversionGoalCampaignConfigsRequest",
    "MutateConversionGoalCampaignConfigsResponse",
    "CallConversion",
    "CallConversionResult",
    "CartData",
    "ClickConversion",
    "ClickConversionResult",
    "CustomVariable",
    "ExternalAttributionData",
    "SessionAttributeKeyValuePair",
    "SessionAttributesKeyValuePairs",
    "UploadCallConversionsRequest",
    "UploadCallConversionsResponse",
    "UploadClickConversionsRequest",
    "UploadClickConversionsResponse",
    "ConversionValueRuleOperation",
    "MutateConversionValueRuleResult",
    "MutateConversionValueRulesRequest",
    "MutateConversionValueRulesResponse",
    "ConversionValueRuleSetOperation",
    "MutateConversionValueRuleSetResult",
    "MutateConversionValueRuleSetsRequest",
    "MutateConversionValueRuleSetsResponse",
    "CustomAudienceOperation",
    "MutateCustomAudienceResult",
    "MutateCustomAudiencesRequest",
    "MutateCustomAudiencesResponse",
    "CustomConversionGoalOperation",
    "MutateCustomConversionGoalResult",
    "MutateCustomConversionGoalsRequest",
    "MutateCustomConversionGoalsResponse",
    "CustomInterestOperation",
    "MutateCustomInterestResult",
    "MutateCustomInterestsRequest",
    "MutateCustomInterestsResponse",
    "CustomerAssetOperation",
    "MutateCustomerAssetResult",
    "MutateCustomerAssetsRequest",
    "MutateCustomerAssetsResponse",
    "CustomerAssetSetOperation",
    "MutateCustomerAssetSetResult",
    "MutateCustomerAssetSetsRequest",
    "MutateCustomerAssetSetsResponse",
    "CustomerClientLinkOperation",
    "MutateCustomerClientLinkRequest",
    "MutateCustomerClientLinkResponse",
    "MutateCustomerClientLinkResult",
    "CustomerConversionGoalOperation",
    "MutateCustomerConversionGoalResult",
    "MutateCustomerConversionGoalsRequest",
    "MutateCustomerConversionGoalsResponse",
    "CustomerCustomizerOperation",
    "MutateCustomerCustomizerResult",
    "MutateCustomerCustomizersRequest",
    "MutateCustomerCustomizersResponse",
    "CustomerLabelOperation",
    "MutateCustomerLabelResult",
    "MutateCustomerLabelsRequest",
    "MutateCustomerLabelsResponse",
    "ConfigureCustomerLifecycleGoalsRequest",
    "ConfigureCustomerLifecycleGoalsResponse",
    "ConfigureCustomerLifecycleGoalsResult",
    "CustomerLifecycleGoalOperation",
    "CustomerManagerLinkOperation",
    "MoveManagerLinkRequest",
    "MoveManagerLinkResponse",
    "MutateCustomerManagerLinkRequest",
    "MutateCustomerManagerLinkResponse",
    "MutateCustomerManagerLinkResult",
    "CustomerNegativeCriterionOperation",
    "MutateCustomerNegativeCriteriaRequest",
    "MutateCustomerNegativeCriteriaResponse",
    "MutateCustomerNegativeCriteriaResult",
    "CreateCustomerClientRequest",
    "CreateCustomerClientResponse",
    "CustomerOperation",
    "ListAccessibleCustomersRequest",
    "ListAccessibleCustomersResponse",
    "MutateCustomerRequest",
    "MutateCustomerResponse",
    "MutateCustomerResult",
    "CustomerSkAdNetworkConversionValueSchemaOperation",
    "MutateCustomerSkAdNetworkConversionValueSchemaRequest",
    "MutateCustomerSkAdNetworkConversionValueSchemaResponse",
    "MutateCustomerSkAdNetworkConversionValueSchemaResult",
    "CustomerUserAccessInvitationOperation",
    "MutateCustomerUserAccessInvitationRequest",
    "MutateCustomerUserAccessInvitationResponse",
    "MutateCustomerUserAccessInvitationResult",
    "CustomerUserAccessOperation",
    "MutateCustomerUserAccessRequest",
    "MutateCustomerUserAccessResponse",
    "MutateCustomerUserAccessResult",
    "CustomizerAttributeOperation",
    "MutateCustomizerAttributeResult",
    "MutateCustomizerAttributesRequest",
    "MutateCustomizerAttributesResponse",
    "CreateDataLinkRequest",
    "CreateDataLinkResponse",
    "RemoveDataLinkRequest",
    "RemoveDataLinkResponse",
    "UpdateDataLinkRequest",
    "UpdateDataLinkResponse",
    "ExperimentArmOperation",
    "MutateExperimentArmResult",
    "MutateExperimentArmsRequest",
    "MutateExperimentArmsResponse",
    "CampaignBudgetMapping",
    "EndExperimentRequest",
    "ExperimentOperation",
    "GraduateExperimentRequest",
    "ListExperimentAsyncErrorsRequest",
    "ListExperimentAsyncErrorsResponse",
    "MutateExperimentResult",
    "MutateExperimentsRequest",
    "MutateExperimentsResponse",
    "PromoteExperimentMetadata",
    "PromoteExperimentRequest",
    "ScheduleExperimentMetadata",
    "ScheduleExperimentRequest",
    "GeoTargetConstantSuggestion",
    "SuggestGeoTargetConstantsRequest",
    "SuggestGeoTargetConstantsResponse",
    "GoalOperation",
    "MutateGoalResult",
    "MutateGoalsRequest",
    "MutateGoalsResponse",
    "GetGoogleAdsFieldRequest",
    "SearchGoogleAdsFieldsRequest",
    "SearchGoogleAdsFieldsResponse",
    "GoogleAdsRow",
    "MutateGoogleAdsRequest",
    "MutateGoogleAdsResponse",
    "MutateOperation",
    "MutateOperationResponse",
    "SearchGoogleAdsRequest",
    "SearchGoogleAdsResponse",
    "SearchGoogleAdsStreamRequest",
    "SearchGoogleAdsStreamResponse",
    "SearchSettings",
    "GetIdentityVerificationRequest",
    "GetIdentityVerificationResponse",
    "IdentityVerification",
    "IdentityVerificationProgress",
    "IdentityVerificationRequirement",
    "StartIdentityVerificationRequest",
    "ListInvoicesRequest",
    "ListInvoicesResponse",
    "KeywordPlanAdGroupKeywordOperation",
    "MutateKeywordPlanAdGroupKeywordResult",
    "MutateKeywordPlanAdGroupKeywordsRequest",
    "MutateKeywordPlanAdGroupKeywordsResponse",
    "KeywordPlanAdGroupOperation",
    "MutateKeywordPlanAdGroupResult",
    "MutateKeywordPlanAdGroupsRequest",
    "MutateKeywordPlanAdGroupsResponse",
    "KeywordPlanCampaignKeywordOperation",
    "MutateKeywordPlanCampaignKeywordResult",
    "MutateKeywordPlanCampaignKeywordsRequest",
    "MutateKeywordPlanCampaignKeywordsResponse",
    "KeywordPlanCampaignOperation",
    "MutateKeywordPlanCampaignResult",
    "MutateKeywordPlanCampaignsRequest",
    "MutateKeywordPlanCampaignsResponse",
    "AdGroupKeywordSuggestion",
    "BiddableKeyword",
    "CampaignToForecast",
    "CriterionBidModifier",
    "ForecastAdGroup",
    "GenerateAdGroupThemesRequest",
    "GenerateAdGroupThemesResponse",
    "GenerateKeywordForecastMetricsRequest",
    "GenerateKeywordForecastMetricsResponse",
    "GenerateKeywordHistoricalMetricsRequest",
    "GenerateKeywordHistoricalMetricsResponse",
    "GenerateKeywordHistoricalMetricsResult",
    "GenerateKeywordIdeaResponse",
    "GenerateKeywordIdeaResult",
    "GenerateKeywordIdeasRequest",
    "KeywordAndUrlSeed",
    "KeywordForecastMetrics",
    "KeywordSeed",
    "ManualCpcBiddingStrategy",
    "MaximizeClicksBiddingStrategy",
    "MaximizeConversionsBiddingStrategy",
    "SiteSeed",
    "UnusableAdGroup",
    "UrlSeed",
    "KeywordPlanOperation",
    "MutateKeywordPlansRequest",
    "MutateKeywordPlansResponse",
    "MutateKeywordPlansResult",
    "SuggestKeywordThemeConstantsRequest",
    "SuggestKeywordThemeConstantsResponse",
    "LabelOperation",
    "MutateLabelResult",
    "MutateLabelsRequest",
    "MutateLabelsResponse",
    "AppendLeadConversationRequest",
    "AppendLeadConversationResponse",
    "Conversation",
    "ConversationOrError",
    "ProvideLeadFeedbackRequest",
    "ProvideLeadFeedbackResponse",
    "SurveyDissatisfied",
    "SurveySatisfied",
    "AddOfflineUserDataJobOperationsRequest",
    "AddOfflineUserDataJobOperationsResponse",
    "CreateOfflineUserDataJobRequest",
    "CreateOfflineUserDataJobResponse",
    "OfflineUserDataJobOperation",
    "RunOfflineUserDataJobRequest",
    "ListPaymentsAccountsRequest",
    "ListPaymentsAccountsResponse",
    "CreateProductLinkInvitationRequest",
    "CreateProductLinkInvitationResponse",
    "RemoveProductLinkInvitationRequest",
    "RemoveProductLinkInvitationResponse",
    "UpdateProductLinkInvitationRequest",
    "UpdateProductLinkInvitationResponse",
    "CreateProductLinkRequest",
    "CreateProductLinkResponse",
    "RemoveProductLinkRequest",
    "RemoveProductLinkResponse",
    "AdvancedProductTargeting",
    "AudienceTargeting",
    "CampaignDuration",
    "ConversionRateSuggestion",
    "EffectiveFrequencyBreakdown",
    "EffectiveFrequencyLimit",
    "Forecast",
    "ForecastMetricOptions",
    "FrequencyCap",
    "GenerateConversionRatesRequest",
    "GenerateConversionRatesResponse",
    "GenerateReachForecastRequest",
    "GenerateReachForecastResponse",
    "ListPlannableLocationsRequest",
    "ListPlannableLocationsResponse",
    "ListPlannableProductsRequest",
    "ListPlannableProductsResponse",
    "ListPlannableUserInterestsRequest",
    "ListPlannableUserInterestsResponse",
    "ListPlannableUserListsRequest",
    "ListPlannableUserListsResponse",
    "OnTargetAudienceMetrics",
    "PlannableLocation",
    "PlannableTargeting",
    "PlannableUserInterest",
    "PlannableUserList",
    "PlannableUserListMetadata",
    "PlannedProduct",
    "PlannedProductForecast",
    "PlannedProductReachForecast",
    "ProductMetadata",
    "ReachCurve",
    "ReachForecast",
    "SurfaceTargeting",
    "SurfaceTargetingCombinations",
    "TargetFrequencySettings",
    "Targeting",
    "YouTubeSelectLineUp",
    "YouTubeSelectSettings",
    "ApplyRecommendationOperation",
    "ApplyRecommendationRequest",
    "ApplyRecommendationResponse",
    "ApplyRecommendationResult",
    "DismissRecommendationRequest",
    "DismissRecommendationResponse",
    "GenerateRecommendationsRequest",
    "GenerateRecommendationsResponse",
    "MutateRecommendationSubscriptionRequest",
    "MutateRecommendationSubscriptionResponse",
    "MutateRecommendationSubscriptionResult",
    "RecommendationSubscriptionOperation",
    "MutateRemarketingActionResult",
    "MutateRemarketingActionsRequest",
    "MutateRemarketingActionsResponse",
    "RemarketingActionOperation",
    "AssetGroupIdentifier",
    "GenerateShareablePreviewsRequest",
    "GenerateShareablePreviewsResponse",
    "ShareablePreview",
    "ShareablePreviewOrError",
    "ShareablePreviewResult",
    "MutateSharedCriteriaRequest",
    "MutateSharedCriteriaResponse",
    "MutateSharedCriterionResult",
    "SharedCriterionOperation",
    "MutateSharedSetResult",
    "MutateSharedSetsRequest",
    "MutateSharedSetsResponse",
    "SharedSetOperation",
    "GetSmartCampaignStatusRequest",
    "GetSmartCampaignStatusResponse",
    "MutateSmartCampaignSettingResult",
    "MutateSmartCampaignSettingsRequest",
    "MutateSmartCampaignSettingsResponse",
    "SmartCampaignEligibleDetails",
    "SmartCampaignEndedDetails",
    "SmartCampaignNotEligibleDetails",
    "SmartCampaignPausedDetails",
    "SmartCampaignRemovedDetails",
    "SmartCampaignSettingOperation",
    "SmartCampaignSuggestionInfo",
    "SuggestKeywordThemesRequest",
    "SuggestKeywordThemesResponse",
    "SuggestSmartCampaignAdRequest",
    "SuggestSmartCampaignAdResponse",
    "SuggestSmartCampaignBudgetOptionsRequest",
    "SuggestSmartCampaignBudgetOptionsResponse",
    "RegenerateShareableLinkIdRequest",
    "RegenerateShareableLinkIdResponse",
    "HotelAssetSuggestion",
    "HotelImageAsset",
    "HotelTextAsset",
    "SuggestTravelAssetsRequest",
    "SuggestTravelAssetsResponse",
    "UploadUserDataRequest",
    "UploadUserDataResponse",
    "UserDataOperation",
    "MutateUserListCustomerTypeResult",
    "MutateUserListCustomerTypesRequest",
    "MutateUserListCustomerTypesResponse",
    "UserListCustomerTypeOperation",
    "MutateUserListResult",
    "MutateUserListsRequest",
    "MutateUserListsResponse",
    "UserListOperation",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "OfflineEventUploadClientEnum",
    },
)


class OfflineEventUploadClientEnum(proto.Message):
    r"""All possible clients for an offline upload event."""

    class OfflineEventUploadClient(proto.Enum):
        r"""Type of client.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            GOOGLE_ADS_API (2):
                Google Ads API.
            GOOGLE_ADS_WEB_CLIENT (3):
                Google Ads web client, which could include
                multiple sources like Ads UI, SFTP, etc.
            ADS_DATA_CONNECTOR (4):
                Connection platform.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        GOOGLE_ADS_API = 2
        GOOGLE_ADS_WEB_CLIENT = 3
        ADS_DATA_CONNECTOR = 4


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.resources.types import (
    product_link as gagr_product_link,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "CreateProductLinkRequest",
        "CreateProductLinkResponse",
        "RemoveProductLinkRequest",
        "RemoveProductLinkResponse",
    },
)


class CreateProductLinkRequest(proto.Message):
    r"""Request message for
    [ProductLinkService.CreateProductLink][google.ads.googleads.v22.services.ProductLinkService.CreateProductLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer for which
            the product link is created.
        product_link (google.ads.googleads.v22.resources.types.ProductLink):
            Required. The product link to be created.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    product_link: gagr_product_link.ProductLink = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_product_link.ProductLink,
    )


class CreateProductLinkResponse(proto.Message):
    r"""Response message for
    [ProductLinkService.CreateProductLink][google.ads.googleads.v22.services.ProductLinkService.CreateProductLink].

    Attributes:
        resource_name (str):
            Returned for successful operations. Resource
            name of the product link.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class RemoveProductLinkRequest(proto.Message):
    r"""Request message for
    [ProductLinkService.RemoveProductLink][google.ads.googleads.v22.services.ProductLinkService.RemoveProductLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer being
            modified.
        resource_name (str):
            Required. Remove operation: A resource name for the product
            link to remove is expected, in this format:

            ``customers/{customer_id}/productLinks/{product_link_id}``
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    resource_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class RemoveProductLinkResponse(proto.Message):
    r"""Response message for product link removal.

    Attributes:
        resource_name (str):
            Result for the remove request.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds an asset for use in dynamic remarketing."""


import argparse
from datetime import datetime
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.asset_types import (
    DynamicEducationAsset,
)
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.resources.types.asset_set import AssetSet
from google.ads.googleads.v22.resources.types.asset_set_asset import (
    AssetSetAsset,
)
from google.ads.googleads.v22.resources.types.campaign_asset_set import (
    CampaignAssetSet,
)
from google.ads.googleads.v22.services.services.asset_service import (
    AssetServiceClient,
)
from google.ads.googleads.v22.services.types.asset_service import (
    AssetOperation,
    MutateAssetsResponse,
)
from google.ads.googleads.v22.services.services.asset_set_service import (
    AssetSetServiceClient,
)
from google.ads.googleads.v22.services.types.asset_set_service import (
    AssetSetOperation,
    MutateAssetSetsResponse,
)
from google.ads.googleads.v22.services.services.asset_set_asset_service import (
    AssetSetAssetServiceClient,
)
from google.ads.googleads.v22.services.types.asset_set_asset_service import (
    AssetSetAssetOperation,
    MutateAssetSetAssetsResponse,
)
from google.ads.googleads.v22.services.services.campaign_asset_set_service import (
    CampaignAssetSetServiceClient,
)
from google.ads.googleads.v22.services.types.campaign_asset_set_service import (
    CampaignAssetSetOperation,
    MutateCampaignAssetSetsResponse,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)


def main(client: GoogleAdsClient, customer_id: str, campaign_id: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        campaign_id: the ID for a campaign of a type that supports dynamic
            remarketing, such as Display.
    """
    asset_resource_name: str = create_asset(client, customer_id)
    asset_set_resource_name: str = create_asset_set(client, customer_id)
    add_assets_to_asset_set(
        client, asset_resource_name, asset_set_resource_name, customer_id
    )
    link_asset_set_to_campaign(
        client, asset_set_resource_name, customer_id, campaign_id
    )


# [START add_asset]
def create_asset(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates a DynamicEducationAsset.

    See https://support.google.com/google-ads/answer/6053288?#zippy=%2Ceducation
    for a detailed explanation of the field format.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        The resource name for an asset.
    """
    # Creates an operation to add the asset.
    operation: AssetOperation = client.get_type("AssetOperation")
    asset: Asset = operation.create
    # The final_urls list must not be empty
    asset.final_urls.append("https://www.example.com")
    education_asset: DynamicEducationAsset = asset.dynamic_education_asset
    # Defines meta-information about the school and program.
    education_asset.school_name = "The University of Unknown"
    education_asset.address = "Building 1, New York, 12345, USA"
    education_asset.program_name = "BSc. Computer Science"
    education_asset.subject = "Computer Science"
    education_asset.program_description = "Slinging code for fun and profit!"
    # Sets up the program ID which is the ID that should be specified in the
    # tracking pixel.
    education_asset.program_id = "bsc-cs-uofu"
    # Sets up the location ID which may additionally be specified in the
    # tracking pixel.
    education_asset.location_id = "nyc"
    education_asset.image_url = "https://gaagl.page.link/Eit5"
    education_asset.android_app_link = (
        "android-app://com.example.android/http/example.com/gizmos?1234"
    )
    education_asset.ios_app_link = "exampleApp://content/page"
    education_asset.ios_app_store_id = 123

    asset_service: AssetServiceClient = client.get_service("AssetService")
    response: MutateAssetsResponse = asset_service.mutate_assets(
        customer_id=customer_id, operations=[operation]
    )
    resource_name: str = response.results[0].resource_name
    print(
        f"Created a dynamic education asset with resource name '{resource_name}'"
    )

    return resource_name
    # [END add_asset]


# [START add_asset_set]
def create_asset_set(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates an AssetSet.

    The AssetSet will be used to link the dynamic remarketing assets to a
    campaign.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        The resource name for an asset set.
    """
    # Creates an operation to create the asset set.
    operation: AssetSetOperation = client.get_type("AssetSetOperation")
    asset_set: AssetSet = operation.create
    asset_set.name = f"My dynamic remarketing assets {datetime.now()}"
    asset_set.type_ = client.enums.AssetSetTypeEnum.DYNAMIC_EDUCATION

    asset_set_service: AssetSetServiceClient = client.get_service(
        "AssetSetService"
    )
    response: MutateAssetSetsResponse = asset_set_service.mutate_asset_sets(
        customer_id=customer_id, operations=[operation]
    )
    resource_name: str = response.results[0].resource_name
    print(f"Created asset set with resource name '{resource_name}'")

    return resource_name
    # [END add_asset_set]


# [START add_asset_set_asset]
def add_assets_to_asset_set(
    client: GoogleAdsClient,
    asset_resource_name: str,
    asset_set_resource_name: str,
    customer_id: str,
) -> None:
    """Adds an Asset to an AssetSet by creating an AssetSetAsset link.

    Args:
        client: an initialized GoogleAdsClient instance.
        asset_set_resource_name; the resource name for an asset set.
        asset_resource_name; the resource name for an asset.
        customer_id: a client customer ID.
    """
    # Creates an operation to add the asset set asset.
    operation: AssetSetAssetOperation = client.get_type(
        "AssetSetAssetOperation"
    )
    asset_set_asset: AssetSetAsset = operation.create
    asset_set_asset.asset = asset_resource_name
    asset_set_asset.asset_set = asset_set_resource_name

    asset_set_asset_service: AssetSetAssetServiceClient = client.get_service(
        "AssetSetAssetService"
    )
    # Note this is the point that the API will enforce uniqueness of the
    # DynamicEducationAsset.program_id field. You can have any number of assets
    # with the same program ID, however, only one asset is allowed per asset set
    # with the same program ID.
    response: MutateAssetSetAssetsResponse = (
        asset_set_asset_service.mutate_asset_set_assets(
            customer_id=customer_id, operations=[operation]
        )
    )
    resource_name: str = response.results[0].resource_name
    print(f"Created asset set asset link with resource name '{resource_name}'")
    # [END add_asset_set_asset]


# [START add_campaign_asset_set]
def link_asset_set_to_campaign(
    client: GoogleAdsClient,
    asset_set_resource_name: str,
    customer_id: str,
    campaign_id: str,
) -> None:
    """Creates a CampaignAssetSet.

    The CampaignAssetSet represents the link between an AssetSet and a Campaign.

    Args:
        client: an initialized GoogleAdsClient instance.
        asset_set_resource_name; the resource name for an asset set.
        customer_id: a client customer ID.
        campaign_id: the ID for a campaign of a type that supports dynamic
            remarketing, such as Display.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    # Creates an operation to add the campaign asset set.
    operation: CampaignAssetSetOperation = client.get_type(
        "CampaignAssetSetOperation"
    )
    campaign_asset_set: CampaignAssetSet = operation.create
    campaign_asset_set.campaign = googleads_service.campaign_path(
        customer_id, campaign_id
    )
    campaign_asset_set.asset_set = asset_set_resource_name

    campaign_asset_set_service: CampaignAssetSetServiceClient = (
        client.get_service("CampaignAssetSetService")
    )
    response: MutateCampaignAssetSetsResponse = (
        campaign_asset_set_service.mutate_campaign_asset_sets(
            customer_id=customer_id, operations=[operation]
        )
    )
    resource_name: str = response.results[0].resource_name
    print(f"Created a campaign asset set with resource name '{resource_name}'")
    # [END add_campaign_asset_set]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Adds an asset for use in dynamic remarketing."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    # The following argument(s) are optional.
    parser.add_argument(
        "-i",
        "--campaign_id",
        type=str,
        required=True,
        help="The campaign ID. Specify a campaign type which supports dynamic "
        "remarketing, such as Display.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.campaign_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import manager_link_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerManagerLink",
    },
)


class CustomerManagerLink(proto.Message):
    r"""Represents customer-manager link relationship.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. Name of the resource. CustomerManagerLink
            resource names have the form:
            ``customers/{customer_id}/customerManagerLinks/{manager_customer_id}~{manager_link_id}``
        manager_customer (str):
            Output only. The manager customer linked to
            the customer.

            This field is a member of `oneof`_ ``_manager_customer``.
        manager_link_id (int):
            Output only. ID of the customer-manager link.
            This field is read only.

            This field is a member of `oneof`_ ``_manager_link_id``.
        status (google.ads.googleads.v20.enums.types.ManagerLinkStatusEnum.ManagerLinkStatus):
            Status of the link between the customer and
            the manager.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    manager_customer: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    manager_link_id: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    status: manager_link_status.ManagerLinkStatusEnum.ManagerLinkStatus = (
        proto.Field(
            proto.ENUM,
            number=5,
            enum=manager_link_status.ManagerLinkStatusEnum.ManagerLinkStatus,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "ExpandedLandingPageView",
    },
)


class ExpandedLandingPageView(proto.Message):
    r"""A landing page view with metrics aggregated at the expanded
    final URL level.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the expanded landing page
            view. Expanded landing page view resource names have the
            form:

            ``customers/{customer_id}/expandedLandingPageViews/{expanded_final_url_fingerprint}``
        expanded_final_url (str):
            Output only. The final URL that clicks are
            directed to.

            This field is a member of `oneof`_ ``_expanded_final_url``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    expanded_final_url: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetGroupAssetErrorEnum",
    },
)


class AssetGroupAssetErrorEnum(proto.Message):
    r"""Container for enum describing possible asset group asset
    errors.

    """

    class AssetGroupAssetError(proto.Enum):
        r"""Enum describing possible asset group asset errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            DUPLICATE_RESOURCE (2):
                Cannot add duplicated asset group asset.
            EXPANDABLE_TAGS_NOT_ALLOWED_IN_DESCRIPTION (3):
                Expandable tags are not allowed in
                description assets.
            AD_CUSTOMIZER_NOT_SUPPORTED (4):
                Ad customizers are not supported in
                assetgroup's text assets.
            HOTEL_PROPERTY_ASSET_NOT_LINKED_TO_CAMPAIGN (5):
                Cannot add a HotelPropertyAsset to an AssetGroup that isn't
                linked to the parent campaign's hotel_property_asset_set
                field.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_RESOURCE = 2
        EXPANDABLE_TAGS_NOT_ALLOWED_IN_DESCRIPTION = 3
        AD_CUSTOMIZER_NOT_SUPPORTED = 4
        HOTEL_PROPERTY_ASSET_NOT_LINKED_TO_CAMPAIGN = 5


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example adds demographic target criteria to an ad group, one as
positive ad group criterion and one as negative ad group criterion. To
create ad groups, run add_ad_groups.py."""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.ad_group_criterion import (
    AdGroupCriterion,
)
from google.ads.googleads.v22.services.services.ad_group_criterion_service import (
    AdGroupCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_criterion_service import (
    AdGroupCriterionOperation,
    MutateAdGroupCriteriaResponse,
)


def main(client: GoogleAdsClient, customer_id: str, ad_group_id: str) -> None:
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )
    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )

    ad_group_resource_name: str = ad_group_service.ad_group_path(
        customer_id, ad_group_id
    )
    # Create a positive ad group criterion for the gender MALE.
    gender_ad_group_criterion_operation: AdGroupCriterionOperation = (
        client.get_type("AdGroupCriterionOperation")
    )
    gender_ad_group_criterion: AdGroupCriterion = (
        gender_ad_group_criterion_operation.create
    )
    gender_ad_group_criterion.ad_group = ad_group_resource_name
    gender_ad_group_criterion.gender.type_ = client.enums.GenderTypeEnum.MALE

    # Create a negative ad group criterion for age range of 18 to 24.
    age_range_ad_group_criterion_operation: AdGroupCriterionOperation = (
        client.get_type("AdGroupCriterionOperation")
    )
    age_range_ad_group_criterion: AdGroupCriterion = (
        age_range_ad_group_criterion_operation.create
    )
    age_range_ad_group_criterion.ad_group = ad_group_resource_name
    age_range_ad_group_criterion.negative = True
    age_range_ad_group_criterion.age_range.type_ = (
        client.enums.AgeRangeTypeEnum.AGE_RANGE_18_24
    )

    # Add two ad group criteria
    ad_group_criterion_response: MutateAdGroupCriteriaResponse = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id,
            operations=[
                gender_ad_group_criterion_operation,
                age_range_ad_group_criterion_operation,
            ],
        )
    )

    for result in ad_group_criterion_response.results:
        print(f"Created ad group criterion '{result.resource_name}'.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Adds demographic targeting criteria to the provided"
            " ad group, for the specified customer."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a", "--ad_group_id", type=str, required=True, help="The ad group ID."
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.ad_group_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "TopicConstant",
    },
)


class TopicConstant(proto.Message):
    r"""Use topics to target or exclude placements in the Google
    Display Network based on the category into which the placement
    falls (for example, "Pets & Animals/Pets/Dogs").


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the topic constant. topic
            constant resource names have the form:

            ``topicConstants/{topic_id}``
        id (int):
            Output only. The ID of the topic.

            This field is a member of `oneof`_ ``_id``.
        topic_constant_parent (str):
            Output only. Resource name of parent of the
            topic constant.

            This field is a member of `oneof`_ ``_topic_constant_parent``.
        path (MutableSequence[str]):
            Output only. The category to target or
            exclude. Each subsequent element in the array
            describes a more specific sub-category. For
            example, {"Pets & Animals", "Pets", "Dogs"}
            represents the "Pets & Animals/Pets/Dogs"
            category. List of available topic categories at
            https://developers.google.com/google-ads/api/reference/data/verticals
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    topic_constant_parent: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    path: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import customizer_attribute_status
from google.ads.googleads.v21.enums.types import customizer_attribute_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "CustomizerAttribute",
    },
)


class CustomizerAttribute(proto.Message):
    r"""A customizer attribute.
    Use CustomerCustomizer, CampaignCustomizer, AdGroupCustomizer,
    or AdGroupCriterionCustomizer to associate a customizer
    attribute and set its value at the customer, campaign, ad group,
    or ad group criterion level, respectively.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the customizer attribute.
            Customizer Attribute resource names have the form:

            ``customers/{customer_id}/customizerAttributes/{customizer_attribute_id}``
        id (int):
            Output only. The ID of the customizer
            attribute.
        name (str):
            Required. Immutable. Name of the customizer
            attribute. Required. It must have a minimum
            length of 1 and maximum length of 40. Name of an
            enabled customizer attribute must be unique
            (case insensitive).
        type_ (google.ads.googleads.v21.enums.types.CustomizerAttributeTypeEnum.CustomizerAttributeType):
            Immutable. The type of the customizer
            attribute.
        status (google.ads.googleads.v21.enums.types.CustomizerAttributeStatusEnum.CustomizerAttributeStatus):
            Output only. The status of the customizer
            attribute.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    type_: (
        customizer_attribute_type.CustomizerAttributeTypeEnum.CustomizerAttributeType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=customizer_attribute_type.CustomizerAttributeTypeEnum.CustomizerAttributeType,
    )
    status: (
        customizer_attribute_status.CustomizerAttributeStatusEnum.CustomizerAttributeStatus
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=customizer_attribute_status.CustomizerAttributeStatusEnum.CustomizerAttributeStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v19.resources.types import asset as gagr_asset
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "MutateAssetsRequest",
        "AssetOperation",
        "MutateAssetsResponse",
        "MutateAssetResult",
    },
)


class MutateAssetsRequest(proto.Message):
    r"""Request message for
    [AssetService.MutateAssets][google.ads.googleads.v19.services.AssetService.MutateAssets]

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose assets
            are being modified.
        operations (MutableSequence[google.ads.googleads.v19.services.types.AssetOperation]):
            Required. The list of operations to perform
            on individual assets.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        response_content_type (google.ads.googleads.v19.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["AssetOperation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="AssetOperation",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=5,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class AssetOperation(proto.Message):
    r"""A single operation to create an asset. Supported asset types
    are YoutubeVideoAsset, MediaBundleAsset, ImageAsset,
    LeadFormAsset, LocationAsset, and ImageAsset. TextAsset can be
    created with an Ad inline, but it can also be created apart from
    an Ad like other assets.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v19.resources.types.Asset):
            Create operation: No resource name is
            expected for the new asset.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v19.resources.types.Asset):
            Update operation: The asset is expected to have a valid
            resource name in this format:

            ``customers/{customer_id}/assets/{asset_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=3,
        message=field_mask_pb2.FieldMask,
    )
    create: gagr_asset.Asset = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_asset.Asset,
    )
    update: gagr_asset.Asset = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_asset.Asset,
    )


class MutateAssetsResponse(proto.Message):
    r"""Response message for an asset mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v19.services.types.MutateAssetResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateAssetResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="MutateAssetResult",
    )


class MutateAssetResult(proto.Message):
    r"""The result for the asset mutate.

    Attributes:
        resource_name (str):
            The resource name returned for successful
            operations.
        asset (google.ads.googleads.v19.resources.types.Asset):
            The mutated asset with only mutable fields after mutate. The
            field will only be returned when response_content_type is
            set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset: gagr_asset.Asset = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_asset.Asset,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "KeywordPlanCompetitionLevelEnum",
    },
)


class KeywordPlanCompetitionLevelEnum(proto.Message):
    r"""Container for enumeration of keyword competition levels. The
    competition level indicates how competitive ad placement is for
    a keyword and is determined by the number of advertisers bidding
    on that keyword relative to all keywords across Google. The
    competition level can depend on the location and Search Network
    targeting options you've selected.

    """

    class KeywordPlanCompetitionLevel(proto.Enum):
        r"""Competition level of a keyword.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            LOW (2):
                Low competition. The Competition Index range for this is [0,
                33].
            MEDIUM (3):
                Medium competition. The Competition Index range for this is
                [34, 66].
            HIGH (4):
                High competition. The Competition Index range for this is
                [67, 100].
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        LOW = 2
        MEDIUM = 3
        HIGH = 4


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "RequestErrorEnum",
    },
)


class RequestErrorEnum(proto.Message):
    r"""Container for enum describing possible request errors."""

    class RequestError(proto.Enum):
        r"""Enum describing possible request errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            RESOURCE_NAME_MISSING (3):
                Resource name is required for this request.
            RESOURCE_NAME_MALFORMED (4):
                Resource name provided is malformed.
            BAD_RESOURCE_ID (17):
                Resource name provided is malformed.
            INVALID_CUSTOMER_ID (16):
                Customer ID is invalid.
            OPERATION_REQUIRED (5):
                Mutate operation should have either create,
                update, or remove specified.
            RESOURCE_NOT_FOUND (6):
                Requested resource not found.
            INVALID_PAGE_TOKEN (7):
                Next page token specified in user request is
                invalid.
            EXPIRED_PAGE_TOKEN (8):
                Next page token specified in user request has
                expired.
            INVALID_PAGE_SIZE (22):
                Page size specified in user request is
                invalid.
            PAGE_SIZE_NOT_SUPPORTED (40):
                Setting the page size is not supported, and
                will be unavailable in a future version.
            REQUIRED_FIELD_MISSING (9):
                Required field is missing.
            IMMUTABLE_FIELD (11):
                The field cannot be modified because it's
                immutable. It's also possible that the field can
                be modified using 'create' operation but not
                'update'.
            TOO_MANY_MUTATE_OPERATIONS (13):
                Received too many entries in request.
            CANNOT_BE_EXECUTED_BY_MANAGER_ACCOUNT (14):
                Request cannot be executed by a manager
                account.
            CANNOT_MODIFY_FOREIGN_FIELD (15):
                Mutate request was attempting to modify a
                readonly field. For instance, Budget fields can
                be requested for Ad Group, but are read-only for
                adGroups:mutate.
            INVALID_ENUM_VALUE (18):
                Enum value is not permitted.
            DEVELOPER_TOKEN_PARAMETER_MISSING (19):
                The developer-token parameter is required for
                all requests.
            LOGIN_CUSTOMER_ID_PARAMETER_MISSING (20):
                The login-customer-id parameter is required
                for this request.
            VALIDATE_ONLY_REQUEST_HAS_PAGE_TOKEN (21):
                page_token is set in the validate only request
            CANNOT_RETURN_SUMMARY_ROW_FOR_REQUEST_WITHOUT_METRICS (29):
                return_summary_row cannot be enabled if request did not
                select any metrics field.
            CANNOT_RETURN_SUMMARY_ROW_FOR_VALIDATE_ONLY_REQUESTS (30):
                return_summary_row should not be enabled for validate only
                requests.
            INCONSISTENT_RETURN_SUMMARY_ROW_VALUE (31):
                return_summary_row parameter value should be the same
                between requests with page_token field set and their
                original request.
            TOTAL_RESULTS_COUNT_NOT_ORIGINALLY_REQUESTED (32):
                The total results count cannot be returned if
                it was not requested in the original request.
            RPC_DEADLINE_TOO_SHORT (33):
                Deadline specified by the client was too
                short.
            UNSUPPORTED_VERSION (38):
                This API version has been sunset and is no
                longer supported.
            CLOUD_PROJECT_NOT_FOUND (39):
                The Google Cloud project in the request was
                not found.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        RESOURCE_NAME_MISSING = 3
        RESOURCE_NAME_MALFORMED = 4
        BAD_RESOURCE_ID = 17
        INVALID_CUSTOMER_ID = 16
        OPERATION_REQUIRED = 5
        RESOURCE_NOT_FOUND = 6
        INVALID_PAGE_TOKEN = 7
        EXPIRED_PAGE_TOKEN = 8
        INVALID_PAGE_SIZE = 22
        PAGE_SIZE_NOT_SUPPORTED = 40
        REQUIRED_FIELD_MISSING = 9
        IMMUTABLE_FIELD = 11
        TOO_MANY_MUTATE_OPERATIONS = 13
        CANNOT_BE_EXECUTED_BY_MANAGER_ACCOUNT = 14
        CANNOT_MODIFY_FOREIGN_FIELD = 15
        INVALID_ENUM_VALUE = 18
        DEVELOPER_TOKEN_PARAMETER_MISSING = 19
        LOGIN_CUSTOMER_ID_PARAMETER_MISSING = 20
        VALIDATE_ONLY_REQUEST_HAS_PAGE_TOKEN = 21
        CANNOT_RETURN_SUMMARY_ROW_FOR_REQUEST_WITHOUT_METRICS = 29
        CANNOT_RETURN_SUMMARY_ROW_FOR_VALIDATE_ONLY_REQUESTS = 30
        INCONSISTENT_RETURN_SUMMARY_ROW_VALUE = 31
        TOTAL_RESULTS_COUNT_NOT_ORIGINALLY_REQUESTED = 32
        RPC_DEADLINE_TOO_SHORT = 33
        UNSUPPORTED_VERSION = 38
        CLOUD_PROJECT_NOT_FOUND = 39


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Demonstrates how to request an exemption for policy violations of a keyword.

Note that the example uses an exemptible policy-violating keyword by default.
If you use a keyword that contains non-exemptible policy violations, they will
not be sent for exemption request, and you will still fail to create a keyword.
If you specify a keyword that doesn't violate any policies, this example will
just add the keyword as usual, similar to what the AddKeywords example does.

Note that once you've requested policy exemption for a keyword, when you send
a request for adding it again, the request will pass like when you add a
non-violating keyword.
"""

import argparse
import sys
from typing import Any, List, Optional, Tuple

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.ad_group_criterion_service import (
    AdGroupCriterionServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_criterion_service import (
    AdGroupCriterionOperation,
)
from google.ads.googleads.v22.common.types.policy import PolicyViolationKey


def main(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    keyword_text: str,
) -> None:
    """Demonstrates how to request an exemption for keyword policy violations.

    Args:
        client: The Google Ads client.
        customer_id: The customer ID for which to add the keyword.
        ad_group_id: The ad group ID to which to add keyword.
        keyword_text: The keyword text to add.
    """

    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )

    googleads_exception: Optional[GoogleAdsException]
    ad_group_criterion_operation: AdGroupCriterionOperation
    (
        googleads_exception,
        ad_group_criterion_operation,
    ) = create_keyword_criterion(
        client,
        ad_group_criterion_service,
        customer_id,
        ad_group_id,
        keyword_text,
    )

    try:
        # Try sending exemption requests for creating a keyword. However, if
        # your keyword contains many policy violations, but not all of them are
        # exemptible, the request will not be sent.
        if googleads_exception is not None:
            exempt_policy_violation_keys: List[PolicyViolationKey] = (
                fetch_exempt_policy_violation_keys(googleads_exception)
            )
            request_exemption(
                customer_id,
                ad_group_criterion_service,
                ad_group_criterion_operation,
                exempt_policy_violation_keys,
            )
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)


def create_keyword_criterion(
    client: GoogleAdsClient,
    ad_group_criterion_service: AdGroupCriterionServiceClient,
    customer_id: str,
    ad_group_id: str,
    keyword_text: str,
) -> Tuple[Optional[GoogleAdsException], AdGroupCriterionOperation]:
    """Attempts to add a keyword criterion to an ad group.

    Args:
        client: The GoogleAds client instance.
        ad_group_criterion_service: The AdGroupCriterionService client instance.
        customer_id: The customer ID for which to add the expanded text ad.
        ad_group_id: The ad group ID to which to add an expanded text ad.
        keyword_text: The keyword text to add.

    Returns:
        The GoogleAdsException that occurred (or None if the operation was
        successful) and the modified operation.
    """
    # Constructs an ad group criterion using the keyword text provided.
    ad_group_criterion_operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    ad_group_criterion: Any = ad_group_criterion_operation.create
    ad_group_criterion.ad_group = client.get_service(
        "AdGroupService"
    ).ad_group_path(customer_id, ad_group_id)
    ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.ENABLED
    ad_group_criterion.keyword.text = keyword_text
    ad_group_criterion.keyword.match_type = (
        client.enums.KeywordMatchTypeEnum.EXACT
    )

    try:
        # Try sending a mutate request to add the keyword.
        response: Any = ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id, operations=[ad_group_criterion_operation]
        )
    except GoogleAdsException as googleads_exception:
        # Return the exception in order to extract keyword violation details.
        return googleads_exception, ad_group_criterion_operation

    # Report that the mutate request was completed successfully.
    print(
        "Added a keyword with resource name "
        f"'{response.results[0].resource_name}'."
    )

    return None, ad_group_criterion_operation


# [START handle_keyword_policy_violations]
def fetch_exempt_policy_violation_keys(
    googleads_exception: GoogleAdsException,
) -> List[PolicyViolationKey]:
    """Collects all policy violation keys that can be exempted.

    Args:
        googleads_exception: The exception to check for policy violation(s).

    Returns:
        A list of policy violation keys.
    """
    exempt_policy_violation_keys: List[PolicyViolationKey] = []

    print("Google Ads failure details:")
    for error in googleads_exception.failure.errors:
        print(f"\t{error.error_code}: {error.message}")

        if (
            error.details is not None
            and error.details.policy_violation_details is not None
        ):
            policy_violation_details = error.details.policy_violation_details
            print(
                "\tPolicy violation details:\n"
                f"\t\tExternal policy name: '{policy_violation_details}'\n"
                "\t\tExternal policy description: "
                f"'{policy_violation_details.external_policy_description}'\n"
                f"\t\tIs exemptible? '{policy_violation_details.is_exemptible}'"
            )

            if (
                policy_violation_details.is_exemptible
                and policy_violation_details.key is not None
            ):
                exempt_policy_violation_keys.append(
                    policy_violation_details.key
                )
                print(
                    f"\t\tPolicy violation key: {policy_violation_details.key}"
                )
                print(
                    f"\t\t\tName: '{policy_violation_details.key.policy_name}'"
                    "\t\t\tViolating text: "
                    f"'{policy_violation_details.key.violating_text}'"
                )
            else:
                print(
                    "No exemption request is sent because your keyword "
                    "contained some non-exemptible policy violations."
                )
                raise googleads_exception
        else:
            print(
                "No exemption request is sent because there are non-policy "
                "related errors thrown."
            )
            raise googleads_exception

    return exempt_policy_violation_keys
    # [END handle_keyword_policy_violations]


# [START handle_keyword_policy_violations_1]
def request_exemption(
    customer_id: str,
    ad_group_criterion_service: AdGroupCriterionServiceClient,
    ad_group_criterion_operation: AdGroupCriterionOperation,
    exempt_policy_violation_keys: List[PolicyViolationKey],
) -> None:
    """Sends exemption requests for creating a keyword.

    Args:
        customer_id: The customer ID for which to add the expanded text ad.
        ad_group_criterion_service: The AdGroupCriterionService client instance.
        ad_group_criterion_operation: The AdGroupCriterionOperation for which
            to request exemption.
        exempt_policy_violation_keys: The exemptible policy violation keys.
    """
    print(
        "Attempting to add a keyword again by requesting exemption for its "
        "policy violations."
    )
    ad_group_criterion_operation.exempt_policy_violation_keys.extend(
        exempt_policy_violation_keys
    )
    response: Any = ad_group_criterion_service.mutate_ad_group_criteria(
        customer_id=customer_id, operations=[ad_group_criterion_operation]
    )
    print(
        "Successfully added a keyword with resource name "
        f"'{response.results[0].resource_name}' by requesting a policy "
        "violation exemption."
    )
    # [END handle_keyword_policy_violations_1]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Demonstrates how to request an exemption for policy "
        "violations of a keyword."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--ad_group_id",
        type=str,
        required=True,
        help="The ad group ID to which to add an expanded text ad.",
    )
    parser.add_argument(
        "-k",
        "--keyword_text",
        type=str,
        required=False,
        default="medication",
        help="Specify the keyword text here or use the default keyword "
        "'medication'.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    main(
        googleads_client, args.customer_id, args.ad_group_id, args.keyword_text
    )



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .accessible_bidding_strategy import (
    AccessibleBiddingStrategy,
)
from .account_budget import (
    AccountBudget,
)
from .account_budget_proposal import (
    AccountBudgetProposal,
)
from .account_link import (
    AccountLink,
    ThirdPartyAppAnalyticsLinkIdentifier,
)
from .ad import (
    Ad,
)
from .ad_group import (
    AdGroup,
)
from .ad_group_ad import (
    AdGroupAd,
    AdGroupAdAssetAutomationSetting,
    AdGroupAdPolicySummary,
)
from .ad_group_ad_asset_combination_view import (
    AdGroupAdAssetCombinationView,
)
from .ad_group_ad_asset_view import (
    AdGroupAdAssetPolicySummary,
    AdGroupAdAssetView,
)
from .ad_group_ad_label import (
    AdGroupAdLabel,
)
from .ad_group_asset import (
    AdGroupAsset,
)
from .ad_group_asset_set import (
    AdGroupAssetSet,
)
from .ad_group_audience_view import (
    AdGroupAudienceView,
)
from .ad_group_bid_modifier import (
    AdGroupBidModifier,
)
from .ad_group_criterion import (
    AdGroupCriterion,
)
from .ad_group_criterion_customizer import (
    AdGroupCriterionCustomizer,
)
from .ad_group_criterion_label import (
    AdGroupCriterionLabel,
)
from .ad_group_criterion_simulation import (
    AdGroupCriterionSimulation,
)
from .ad_group_customizer import (
    AdGroupCustomizer,
)
from .ad_group_label import (
    AdGroupLabel,
)
from .ad_group_simulation import (
    AdGroupSimulation,
)
from .ad_parameter import (
    AdParameter,
)
from .ad_schedule_view import (
    AdScheduleView,
)
from .age_range_view import (
    AgeRangeView,
)
from .android_privacy_shared_key_google_ad_group import (
    AndroidPrivacySharedKeyGoogleAdGroup,
)
from .android_privacy_shared_key_google_campaign import (
    AndroidPrivacySharedKeyGoogleCampaign,
)
from .android_privacy_shared_key_google_network_type import (
    AndroidPrivacySharedKeyGoogleNetworkType,
)
from .asset import (
    Asset,
    AssetFieldTypePolicySummary,
    AssetPolicySummary,
)
from .asset_field_type_view import (
    AssetFieldTypeView,
)
from .asset_group import (
    AdStrengthActionItem,
    AssetCoverage,
    AssetGroup,
)
from .asset_group_asset import (
    AssetGroupAsset,
)
from .asset_group_listing_group_filter import (
    AssetGroupListingGroupFilter,
    ListingGroupFilterDimension,
    ListingGroupFilterDimensionPath,
)
from .asset_group_product_group_view import (
    AssetGroupProductGroupView,
)
from .asset_group_signal import (
    AssetGroupSignal,
)
from .asset_group_top_combination_view import (
    AssetGroupAssetCombinationData,
    AssetGroupTopCombinationView,
)
from .asset_set import (
    AssetSet,
)
from .asset_set_asset import (
    AssetSetAsset,
)
from .asset_set_type_view import (
    AssetSetTypeView,
)
from .audience import (
    Audience,
)
from .batch_job import (
    BatchJob,
)
from .bidding_data_exclusion import (
    BiddingDataExclusion,
)
from .bidding_seasonality_adjustment import (
    BiddingSeasonalityAdjustment,
)
from .bidding_strategy import (
    BiddingStrategy,
)
from .bidding_strategy_simulation import (
    BiddingStrategySimulation,
)
from .billing_setup import (
    BillingSetup,
)
from .call_view import (
    CallView,
)
from .campaign import (
    Campaign,
)
from .campaign_aggregate_asset_view import (
    CampaignAggregateAssetView,
)
from .campaign_asset import (
    CampaignAsset,
)
from .campaign_asset_set import (
    CampaignAssetSet,
)
from .campaign_audience_view import (
    CampaignAudienceView,
)
from .campaign_bid_modifier import (
    CampaignBidModifier,
)
from .campaign_budget import (
    CampaignBudget,
)
from .campaign_conversion_goal import (
    CampaignConversionGoal,
)
from .campaign_criterion import (
    CampaignCriterion,
)
from .campaign_customizer import (
    CampaignCustomizer,
)
from .campaign_draft import (
    CampaignDraft,
)
from .campaign_group import (
    CampaignGroup,
)
from .campaign_label import (
    CampaignLabel,
)
from .campaign_lifecycle_goal import (
    CampaignLifecycleGoal,
    CustomerAcquisitionGoalSettings,
)
from .campaign_search_term_insight import (
    CampaignSearchTermInsight,
)
from .campaign_shared_set import (
    CampaignSharedSet,
)
from .campaign_simulation import (
    CampaignSimulation,
)
from .carrier_constant import (
    CarrierConstant,
)
from .change_event import (
    ChangeEvent,
)
from .change_status import (
    ChangeStatus,
)
from .channel_aggregate_asset_view import (
    ChannelAggregateAssetView,
)
from .click_view import (
    ClickView,
)
from .combined_audience import (
    CombinedAudience,
)
from .content_criterion_view import (
    ContentCriterionView,
)
from .conversion_action import (
    ConversionAction,
)
from .conversion_custom_variable import (
    ConversionCustomVariable,
)
from .conversion_goal_campaign_config import (
    ConversionGoalCampaignConfig,
)
from .conversion_value_rule import (
    ConversionValueRule,
)
from .conversion_value_rule_set import (
    ConversionValueRuleSet,
)
from .currency_constant import (
    CurrencyConstant,
)
from .custom_audience import (
    CustomAudience,
    CustomAudienceMember,
)
from .custom_conversion_goal import (
    CustomConversionGoal,
)
from .custom_interest import (
    CustomInterest,
    CustomInterestMember,
)
from .customer import (
    CallReportingSetting,
    ConversionTrackingSetting,
    Customer,
    CustomerAgreementSetting,
    GranularInsuranceStatus,
    GranularLicenseStatus,
    LocalServicesSettings,
    RemarketingSetting,
)
from .customer_asset import (
    CustomerAsset,
)
from .customer_asset_set import (
    CustomerAssetSet,
)
from .customer_client import (
    CustomerClient,
)
from .customer_client_link import (
    CustomerClientLink,
)
from .customer_conversion_goal import (
    CustomerConversionGoal,
)
from .customer_customizer import (
    CustomerCustomizer,
)
from .customer_label import (
    CustomerLabel,
)
from .customer_lifecycle_goal import (
    CustomerLifecycleGoal,
)
from .customer_manager_link import (
    CustomerManagerLink,
)
from .customer_negative_criterion import (
    CustomerNegativeCriterion,
)
from .customer_search_term_insight import (
    CustomerSearchTermInsight,
)
from .customer_sk_ad_network_conversion_value_schema import (
    CustomerSkAdNetworkConversionValueSchema,
)
from .customer_user_access import (
    CustomerUserAccess,
)
from .customer_user_access_invitation import (
    CustomerUserAccessInvitation,
)
from .customizer_attribute import (
    CustomizerAttribute,
)
from .data_link import (
    DataLink,
    YoutubeVideoIdentifier,
)
from .detail_placement_view import (
    DetailPlacementView,
)
from .detailed_demographic import (
    DetailedDemographic,
)
from .display_keyword_view import (
    DisplayKeywordView,
)
from .distance_view import (
    DistanceView,
)
from .domain_category import (
    DomainCategory,
)
from .dynamic_search_ads_search_term_view import (
    DynamicSearchAdsSearchTermView,
)
from .expanded_landing_page_view import (
    ExpandedLandingPageView,
)
from .experiment import (
    Experiment,
)
from .experiment_arm import (
    ExperimentArm,
)
from .gender_view import (
    GenderView,
)
from .geo_target_constant import (
    GeoTargetConstant,
)
from .geographic_view import (
    GeographicView,
)
from .google_ads_field import (
    GoogleAdsField,
)
from .group_placement_view import (
    GroupPlacementView,
)
from .hotel_group_view import (
    HotelGroupView,
)
from .hotel_performance_view import (
    HotelPerformanceView,
)
from .hotel_reconciliation import (
    HotelReconciliation,
)
from .income_range_view import (
    IncomeRangeView,
)
from .invoice import (
    Invoice,
)
from .keyword_plan import (
    KeywordPlan,
    KeywordPlanForecastPeriod,
)
from .keyword_plan_ad_group import (
    KeywordPlanAdGroup,
)
from .keyword_plan_ad_group_keyword import (
    KeywordPlanAdGroupKeyword,
)
from .keyword_plan_campaign import (
    KeywordPlanCampaign,
    KeywordPlanGeoTarget,
)
from .keyword_plan_campaign_keyword import (
    KeywordPlanCampaignKeyword,
)
from .keyword_theme_constant import (
    KeywordThemeConstant,
)
from .keyword_view import (
    KeywordView,
)
from .label import (
    Label,
)
from .landing_page_view import (
    LandingPageView,
)
from .language_constant import (
    LanguageConstant,
)
from .lead_form_submission_data import (
    CustomLeadFormSubmissionField,
    LeadFormSubmissionData,
    LeadFormSubmissionField,
)
from .life_event import (
    LifeEvent,
)
from .local_services_employee import (
    Fellowship,
    LocalServicesEmployee,
    Residency,
    UniversityDegree,
)
from .local_services_lead import (
    ContactDetails,
    CreditDetails,
    LocalServicesLead,
    Note,
)
from .local_services_lead_conversation import (
    LocalServicesLeadConversation,
    MessageDetails,
    PhoneCallDetails,
)
from .local_services_verification_artifact import (
    BackgroundCheckVerificationArtifact,
    BusinessRegistrationCheckVerificationArtifact,
    BusinessRegistrationDocument,
    BusinessRegistrationNumber,
    InsuranceVerificationArtifact,
    LicenseVerificationArtifact,
    LocalServicesVerificationArtifact,
)
from .location_view import (
    LocationView,
)
from .managed_placement_view import (
    ManagedPlacementView,
)
from .media_file import (
    MediaAudio,
    MediaBundle,
    MediaFile,
    MediaImage,
    MediaVideo,
)
from .mobile_app_category_constant import (
    MobileAppCategoryConstant,
)
from .mobile_device_constant import (
    MobileDeviceConstant,
)
from .offline_conversion_upload_client_summary import (
    OfflineConversionAlert,
    OfflineConversionError,
    OfflineConversionSummary,
    OfflineConversionUploadClientSummary,
)
from .offline_conversion_upload_conversion_action_summary import (
    OfflineConversionUploadConversionActionSummary,
)
from .offline_user_data_job import (
    OfflineUserDataJob,
    OfflineUserDataJobMetadata,
)
from .operating_system_version_constant import (
    OperatingSystemVersionConstant,
)
from .paid_organic_search_term_view import (
    PaidOrganicSearchTermView,
)
from .parental_status_view import (
    ParentalStatusView,
)
from .payments_account import (
    PaymentsAccount,
)
from .per_store_view import (
    PerStoreView,
)
from .performance_max_placement_view import (
    PerformanceMaxPlacementView,
)
from .product_category_constant import (
    ProductCategoryConstant,
)
from .product_group_view import (
    ProductGroupView,
)
from .product_link import (
    AdvertisingPartnerIdentifier,
    DataPartnerIdentifier,
    GoogleAdsIdentifier,
    MerchantCenterIdentifier,
    ProductLink,
)
from .product_link_invitation import (
    AdvertisingPartnerLinkInvitationIdentifier,
    HotelCenterLinkInvitationIdentifier,
    MerchantCenterLinkInvitationIdentifier,
    ProductLinkInvitation,
)
from .qualifying_question import (
    QualifyingQuestion,
)
from .recommendation import (
    Recommendation,
)
from .recommendation_subscription import (
    RecommendationSubscription,
)
from .remarketing_action import (
    RemarketingAction,
)
from .search_term_view import (
    SearchTermView,
)
from .shared_criterion import (
    SharedCriterion,
)
from .shared_set import (
    SharedSet,
)
from .shopping_performance_view import (
    ShoppingPerformanceView,
)
from .shopping_product import (
    ShoppingProduct,
)
from .smart_campaign_search_term_view import (
    SmartCampaignSearchTermView,
)
from .smart_campaign_setting import (
    SmartCampaignSetting,
)
from .third_party_app_analytics_link import (
    ThirdPartyAppAnalyticsLink,
)
from .topic_constant import (
    TopicConstant,
)
from .topic_view import (
    TopicView,
)
from .travel_activity_group_view import (
    TravelActivityGroupView,
)
from .travel_activity_performance_view import (
    TravelActivityPerformanceView,
)
from .user_interest import (
    UserInterest,
)
from .user_list import (
    UserList,
)
from .user_list_customer_type import (
    UserListCustomerType,
)
from .user_location_view import (
    UserLocationView,
)
from .video import (
    Video,
)
from .webpage_view import (
    WebpageView,
)

__all__ = (
    "AccessibleBiddingStrategy",
    "AccountBudget",
    "AccountBudgetProposal",
    "AccountLink",
    "ThirdPartyAppAnalyticsLinkIdentifier",
    "Ad",
    "AdGroup",
    "AdGroupAd",
    "AdGroupAdAssetAutomationSetting",
    "AdGroupAdPolicySummary",
    "AdGroupAdAssetCombinationView",
    "AdGroupAdAssetPolicySummary",
    "AdGroupAdAssetView",
    "AdGroupAdLabel",
    "AdGroupAsset",
    "AdGroupAssetSet",
    "AdGroupAudienceView",
    "AdGroupBidModifier",
    "AdGroupCriterion",
    "AdGroupCriterionCustomizer",
    "AdGroupCriterionLabel",
    "AdGroupCriterionSimulation",
    "AdGroupCustomizer",
    "AdGroupLabel",
    "AdGroupSimulation",
    "AdParameter",
    "AdScheduleView",
    "AgeRangeView",
    "AndroidPrivacySharedKeyGoogleAdGroup",
    "AndroidPrivacySharedKeyGoogleCampaign",
    "AndroidPrivacySharedKeyGoogleNetworkType",
    "Asset",
    "AssetFieldTypePolicySummary",
    "AssetPolicySummary",
    "AssetFieldTypeView",
    "AdStrengthActionItem",
    "AssetCoverage",
    "AssetGroup",
    "AssetGroupAsset",
    "AssetGroupListingGroupFilter",
    "ListingGroupFilterDimension",
    "ListingGroupFilterDimensionPath",
    "AssetGroupProductGroupView",
    "AssetGroupSignal",
    "AssetGroupAssetCombinationData",
    "AssetGroupTopCombinationView",
    "AssetSet",
    "AssetSetAsset",
    "AssetSetTypeView",
    "Audience",
    "BatchJob",
    "BiddingDataExclusion",
    "BiddingSeasonalityAdjustment",
    "BiddingStrategy",
    "BiddingStrategySimulation",
    "BillingSetup",
    "CallView",
    "Campaign",
    "CampaignAggregateAssetView",
    "CampaignAsset",
    "CampaignAssetSet",
    "CampaignAudienceView",
    "CampaignBidModifier",
    "CampaignBudget",
    "CampaignConversionGoal",
    "CampaignCriterion",
    "CampaignCustomizer",
    "CampaignDraft",
    "CampaignGroup",
    "CampaignLabel",
    "CampaignLifecycleGoal",
    "CustomerAcquisitionGoalSettings",
    "CampaignSearchTermInsight",
    "CampaignSharedSet",
    "CampaignSimulation",
    "CarrierConstant",
    "ChangeEvent",
    "ChangeStatus",
    "ChannelAggregateAssetView",
    "ClickView",
    "CombinedAudience",
    "ContentCriterionView",
    "ConversionAction",
    "ConversionCustomVariable",
    "ConversionGoalCampaignConfig",
    "ConversionValueRule",
    "ConversionValueRuleSet",
    "CurrencyConstant",
    "CustomAudience",
    "CustomAudienceMember",
    "CustomConversionGoal",
    "CustomInterest",
    "CustomInterestMember",
    "CallReportingSetting",
    "ConversionTrackingSetting",
    "Customer",
    "CustomerAgreementSetting",
    "GranularInsuranceStatus",
    "GranularLicenseStatus",
    "LocalServicesSettings",
    "RemarketingSetting",
    "CustomerAsset",
    "CustomerAssetSet",
    "CustomerClient",
    "CustomerClientLink",
    "CustomerConversionGoal",
    "CustomerCustomizer",
    "CustomerLabel",
    "CustomerLifecycleGoal",
    "CustomerManagerLink",
    "CustomerNegativeCriterion",
    "CustomerSearchTermInsight",
    "CustomerSkAdNetworkConversionValueSchema",
    "CustomerUserAccess",
    "CustomerUserAccessInvitation",
    "CustomizerAttribute",
    "DataLink",
    "YoutubeVideoIdentifier",
    "DetailPlacementView",
    "DetailedDemographic",
    "DisplayKeywordView",
    "DistanceView",
    "DomainCategory",
    "DynamicSearchAdsSearchTermView",
    "ExpandedLandingPageView",
    "Experiment",
    "ExperimentArm",
    "GenderView",
    "GeoTargetConstant",
    "GeographicView",
    "GoogleAdsField",
    "GroupPlacementView",
    "HotelGroupView",
    "HotelPerformanceView",
    "HotelReconciliation",
    "IncomeRangeView",
    "Invoice",
    "KeywordPlan",
    "KeywordPlanForecastPeriod",
    "KeywordPlanAdGroup",
    "KeywordPlanAdGroupKeyword",
    "KeywordPlanCampaign",
    "KeywordPlanGeoTarget",
    "KeywordPlanCampaignKeyword",
    "KeywordThemeConstant",
    "KeywordView",
    "Label",
    "LandingPageView",
    "LanguageConstant",
    "CustomLeadFormSubmissionField",
    "LeadFormSubmissionData",
    "LeadFormSubmissionField",
    "LifeEvent",
    "Fellowship",
    "LocalServicesEmployee",
    "Residency",
    "UniversityDegree",
    "ContactDetails",
    "CreditDetails",
    "LocalServicesLead",
    "Note",
    "LocalServicesLeadConversation",
    "MessageDetails",
    "PhoneCallDetails",
    "BackgroundCheckVerificationArtifact",
    "BusinessRegistrationCheckVerificationArtifact",
    "BusinessRegistrationDocument",
    "BusinessRegistrationNumber",
    "InsuranceVerificationArtifact",
    "LicenseVerificationArtifact",
    "LocalServicesVerificationArtifact",
    "LocationView",
    "ManagedPlacementView",
    "MediaAudio",
    "MediaBundle",
    "MediaFile",
    "MediaImage",
    "MediaVideo",
    "MobileAppCategoryConstant",
    "MobileDeviceConstant",
    "OfflineConversionAlert",
    "OfflineConversionError",
    "OfflineConversionSummary",
    "OfflineConversionUploadClientSummary",
    "OfflineConversionUploadConversionActionSummary",
    "OfflineUserDataJob",
    "OfflineUserDataJobMetadata",
    "OperatingSystemVersionConstant",
    "PaidOrganicSearchTermView",
    "ParentalStatusView",
    "PaymentsAccount",
    "PerStoreView",
    "PerformanceMaxPlacementView",
    "ProductCategoryConstant",
    "ProductGroupView",
    "AdvertisingPartnerIdentifier",
    "DataPartnerIdentifier",
    "GoogleAdsIdentifier",
    "MerchantCenterIdentifier",
    "ProductLink",
    "AdvertisingPartnerLinkInvitationIdentifier",
    "HotelCenterLinkInvitationIdentifier",
    "MerchantCenterLinkInvitationIdentifier",
    "ProductLinkInvitation",
    "QualifyingQuestion",
    "Recommendation",
    "RecommendationSubscription",
    "RemarketingAction",
    "SearchTermView",
    "SharedCriterion",
    "SharedSet",
    "ShoppingPerformanceView",
    "ShoppingProduct",
    "SmartCampaignSearchTermView",
    "SmartCampaignSetting",
    "ThirdPartyAppAnalyticsLink",
    "TopicConstant",
    "TopicView",
    "TravelActivityGroupView",
    "TravelActivityPerformanceView",
    "UserInterest",
    "UserList",
    "UserListCustomerType",
    "UserLocationView",
    "Video",
    "WebpageView",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import text_label as gagc_text_label
from google.ads.googleads.v22.enums.types import label_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "Label",
    },
)


class Label(proto.Message):
    r"""A label.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. Name of the resource. Label resource names have
            the form: ``customers/{customer_id}/labels/{label_id}``
        id (int):
            Output only. ID of the label. Read only.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the label.

            This field is required and should not be empty
            when creating a new label.

            The length of this string should be between 1
            and 80, inclusive.

            This field is a member of `oneof`_ ``_name``.
        status (google.ads.googleads.v22.enums.types.LabelStatusEnum.LabelStatus):
            Output only. Status of the label. Read only.
        text_label (google.ads.googleads.v22.common.types.TextLabel):
            A type of label displaying text on a colored
            background.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    status: label_status.LabelStatusEnum.LabelStatus = proto.Field(
        proto.ENUM,
        number=4,
        enum=label_status.LabelStatusEnum.LabelStatus,
    )
    text_label: gagc_text_label.TextLabel = proto.Field(
        proto.MESSAGE,
        number=5,
        message=gagc_text_label.TextLabel,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A gRPC Interceptor that is responsible to augmenting request metadata.

This class is initialized in the GoogleAdsClient and passed into a grpc
intercept_channel whenever a new service is initialized. It intercepts requests
and updates the metadata in order to insert the developer token and
login-customer-id values.
"""

# TODO: Explicitly importing the protobuf package version here should be removed
# once the below issue is resolved, and the protobuf version is added to the
# request user-agent directly by the google-api-core package:
# https://github.com/googleapis/python-api-core/issues/416
from importlib import metadata

try:
    _PROTOBUF_VERSION = metadata.version("protobuf")
except metadata.PackageNotFoundError:
    _PROTOBUF_VERSION = None


from google.protobuf.internal import api_implementation
from grpc import UnaryUnaryClientInterceptor, UnaryStreamClientInterceptor

from .interceptor import Interceptor

# Determine which protobuf implementation is being used.
if api_implementation.Type() == "cpp":
    _PB_IMPL_HEADER = "+c"
elif api_implementation.Type() == "python":
    _PB_IMPL_HEADER = "+n"
else:
    _PB_IMPL_HEADER = ""


class MetadataInterceptor(
    Interceptor, UnaryUnaryClientInterceptor, UnaryStreamClientInterceptor
):
    """An interceptor that appends custom metadata to requests."""

    def __init__(
        self,
        developer_token,
        login_customer_id,
        linked_customer_id=None,
        use_cloud_org_for_api_access=None,
    ):
        """Initialization method for this class.

        Args:
            developer_token: a str developer token.
            login_customer_id: a str specifying a login customer ID.
            linked_customer_id: a str specifying a linked customer ID.
            use_cloud_org_for_api_access: a str specifying whether to use the
                Google Cloud Organization of your Google Cloud project instead
                of developer token to determine your Google Ads API access
                levels. Use this flag only if you are enrolled into a limited
                pilot that supports this configuration
        """
        self.developer_token_meta = ("developer-token", developer_token)
        self.login_customer_id_meta = (
            ("login-customer-id", login_customer_id)
            if login_customer_id
            else None
        )
        self.linked_customer_id_meta = (
            ("linked-customer-id", linked_customer_id)
            if linked_customer_id
            else None
        )
        self.use_cloud_org_for_api_access = use_cloud_org_for_api_access

    def _update_client_call_details_metadata(
        self, client_call_details, metadata
    ):
        """Updates the client call details with additional metadata.

        Args:
            client_call_details: An instance of grpc.ClientCallDetails.
            metadata: Additional metadata defined by GoogleAdsClient.

        Returns:
            An new instance of grpc.ClientCallDetails with additional metadata
            from the GoogleAdsClient.
        """
        client_call_details = self.get_client_call_details_instance(
            client_call_details.method,
            client_call_details.timeout,
            metadata,
            client_call_details.credentials,
        )

        return client_call_details

    def _intercept(self, continuation, client_call_details, request):
        """Generic interceptor used for Unary-Unary and Unary-Stream requests.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """
        if client_call_details.metadata is None:
            metadata = []
        else:
            metadata = list(client_call_details.metadata)

        # If self.use_cloud_org_for_api_access is not True, add the developer
        # token to the request's metadata
        if not self.use_cloud_org_for_api_access:
            metadata.append(self.developer_token_meta)

        if self.login_customer_id_meta:
            metadata.append(self.login_customer_id_meta)

        if self.linked_customer_id_meta:
            metadata.append(self.linked_customer_id_meta)

        # TODO: This logic should be updated or removed once the following is
        # fixed: https://github.com/googleapis/python-api-core/issues/416
        for i, metadatum in enumerate(metadata):
            # Check if the user agent header key is in the current metadatum
            if "x-goog-api-client" in metadatum and _PROTOBUF_VERSION:
                # Convert the tuple to a list so it can be modified.
                metadatum = list(metadatum)
                # Check that "pb" isn't already included in the user agent.
                if "pb" not in metadatum[1]:
                    # Append the protobuf version key value pair to the end of
                    # the string.
                    metadatum[1] += f" pb/{_PROTOBUF_VERSION}{_PB_IMPL_HEADER}"
                    # Convert the metadatum back to a tuple.
                    metadatum = tuple(metadatum)
                    # Splice the metadatum back in its original position in
                    # order to preserve the order of the metadata list.
                    metadata[i] = metadatum
                    # Exit the loop since we already found the user agent.
                    break

        client_call_details = self._update_client_call_details_metadata(
            client_call_details, metadata
        )

        return continuation(client_call_details, request)

    def intercept_unary_unary(self, continuation, client_call_details, request):
        """Intercepts and appends custom metadata for Unary-Unary requests.

        Overrides abstract method defined in grpc.UnaryUnaryClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """
        return self._intercept(continuation, client_call_details, request)

    def intercept_unary_stream(
        self, continuation, client_call_details, request
    ):
        """Intercepts and appends custom metadata to Unary-Stream requests.

        Overrides abstract method defined in grpc.UnaryStreamClientInterceptor.

        Args:
            continuation: a function to continue the request process.
            client_call_details: a grpc._interceptor._ClientCallDetails
                instance containing request metadata.
            request: a SearchGoogleAdsRequest or SearchGoogleAdsStreamRequest
                message class instance.

        Returns:
            A grpc.Call/grpc.Future instance representing a service response.
        """
        return self._intercept(continuation, client_call_details, request)



#!/usr/bin/env python
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This code example adds an ad group asset.

To upload image assets, run misc/upload_image_asset.py.
"""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.ad_group_asset import AdGroupAsset
from google.ads.googleads.v22.services.services.ad_group_asset_service.client import (
    AdGroupAssetServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_asset_service import (
    AdGroupAssetOperation,
    MutateAdGroupAssetResult,
    MutateAdGroupAssetsResponse,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    asset_id: str,
) -> None:
    ad_group_asset_service: AdGroupAssetServiceClient = client.get_service(
        "AdGroupAssetService"
    )
    ad_group_asset_resource_name: str = ad_group_asset_service.asset_path(
        customer_id, asset_id
    )

    ad_group_asset_operation: AdGroupAssetOperation = client.get_type(
        "AdGroupAssetOperation"
    )
    ad_group_asset_set: AdGroupAsset = ad_group_asset_operation.create
    ad_group_asset_set.asset = ad_group_asset_resource_name
    ad_group_asset_set.field_type = client.enums.AssetFieldTypeEnum.AD_IMAGE
    ad_group_asset_set.ad_group = ad_group_asset_service.ad_group_path(
        customer_id, ad_group_id
    )
    response: MutateAdGroupAssetsResponse = (
        ad_group_asset_service.mutate_ad_group_assets(
            customer_id=customer_id, operations=[ad_group_asset_operation]
        )
    )

    result: MutateAdGroupAssetResult
    for result in response.results:
        print(
            f"Created ad group asset with resource name: '{result.resource_name}'"
        )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Updates an ad group for specified customer and ad group "
            "id with the given image asset id."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a", "--ad_group_id", type=str, required=True, help="The ad group ID."
    )
    parser.add_argument(
        "-s",
        "--asset_id",
        type=str,
        required=True,
        help="The asset ID.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(
            googleads_client, args.customer_id, args.ad_group_id, args.asset_id
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)
